@@ -11,7 +11,7 @@
 %% The Original Code is RabbitMQ.
 %%
 %% The Initial Developer of the Original Code is VMware, Inc.
-%% Copyright (c) 2007-2012 VMware, Inc.  All rights reserved.
+%% Copyright (c) 2007-2013 VMware, Inc.  All rights reserved.
 %%
 
 -module(rabbit_auth_backend_ldap_app).
@@ -20,6 +20,13 @@
 -export([start/2, stop/1]).
 
 start(_Type, _StartArgs) ->
+    {ok, Backends} = application:get_env(rabbit, auth_backends),
+    case lists:member(rabbit_auth_backend_ldap, Backends) of
+        true  -> ok;
+        false -> rabbit_log:warning(
+                   "LDAP plugin loaded, but rabbit_auth_backend_ldap is not "
+                   "in the list of auth_backends. LDAP auth will not work.~n")
+    end,
     rabbit_auth_backend_ldap_sup:start_link().
 
 stop(_State) ->
