@@ -11,7 +11,7 @@
 %% The Original Code is RabbitMQ Consistent Hash Exchange.
 %%
 %% The Initial Developer of the Original Code is VMware, Inc.
-%% Copyright (c) 2011-2012 VMware, Inc.  All rights reserved.
+%% Copyright (c) 2011-2013 VMware, Inc.  All rights reserved.
 %%
 
 -module(rabbit_exchange_type_consistent_hash).
@@ -20,7 +20,7 @@
 -behaviour(rabbit_exchange_type).
 
 -export([description/0, serialise_events/0, route/2]).
--export([validate/1, create/2, delete/3, add_binding/3,
+-export([validate/1, create/2, delete/3, policy_changed/3, add_binding/3,
          remove_bindings/3, assert_args_equivalence/2]).
 -export([init/0]).
 
@@ -82,7 +82,15 @@ route(#exchange { name = Name } = X,
 
 validate(_X) -> ok.
 create(_Tx, _X) -> ok.
+delete(transaction, #exchange { name = Name }, _Bs) ->
+    ok = mnesia:write_lock_table(?TABLE),
+    [ok = mnesia:delete_object(?TABLE, R, write) ||
+        R <- mnesia:match_object(
+               ?TABLE, #bucket{source_number = {Name, '_'}, _ = '_'}, write)],
+    ok;
+
 delete(_Tx, _X, _Bs) -> ok.
+policy_changed(_Tx, _X1, _X2) -> ok.
 
 add_binding(transaction, _X,
             #binding { source = S, destination = D, key = K } = B) ->
