@@ -11,7 +11,7 @@
 %% The Original Code is RabbitMQ.
 %%
 %% The Initial Developer of the Original Code is VMware, Inc.
-%% Copyright (c) 2007-2012 VMware, Inc.  All rights reserved.
+%% Copyright (c) 2007-2013 VMware, Inc.  All rights reserved.
 %%
 
 -module(amqp_client_SUITE).
@@ -40,7 +40,7 @@ amqp_uri_parse_test_()                  -> ?RUN([]).
 basic_get_test_()                       -> ?RUN([]).
 basic_get_ipv6_test_()                  -> ?RUN([]).
 basic_return_test_()                    -> ?RUN([]).
-simultaneous_close_test_()              -> ?RUN([repeat, {timeout, 60}]).
+simultaneous_close_test_()              -> ?RUN([repeat]).
 basic_qos_test_()                       -> ?RUN([]).
 basic_recover_test_()                   -> ?RUN([]).
 basic_consume_test_()                   -> ?RUN([]).
@@ -48,6 +48,7 @@ consume_notification_test_()            -> ?RUN([]).
 basic_nack_test_()                      -> ?RUN([]).
 large_content_test_()                   -> ?RUN([]).
 lifecycle_test_()                       -> ?RUN([]).
+direct_no_password_test_()              -> ?RUN([]).
 nowait_exchange_declare_test_()         -> ?RUN([]).
 channel_repeat_open_close_test_()       -> ?RUN([]).
 channel_multi_open_close_test_()        -> ?RUN([]).
@@ -55,24 +56,24 @@ basic_ack_test_()                       -> ?RUN([]).
 basic_ack_call_test_()                  -> ?RUN([]).
 channel_lifecycle_test_()               -> ?RUN([]).
 queue_unbind_test_()                    -> ?RUN([]).
-sync_method_serialization_test_()       -> ?RUN([{timeout, 60}]).
-async_sync_method_serialization_test_() -> ?RUN([{timeout, 60}]).
-sync_async_method_serialization_test_() -> ?RUN([{timeout, 60}]).
-teardown_test_()                        -> ?RUN([repeat, {timeout, 60}]).
+sync_method_serialization_test_()       -> ?RUN([]).
+async_sync_method_serialization_test_() -> ?RUN([]).
+sync_async_method_serialization_test_() -> ?RUN([]).
+teardown_test_()                        -> ?RUN([repeat]).
 rpc_test_()                             -> ?RUN([]).
-pub_and_close_test_()                   -> ?RUN([{timeout, 60}]).
+pub_and_close_test_()                   -> ?RUN([]).
 channel_tune_negotiation_test_()        -> ?RUN([]).
 confirm_test_()                         -> ?RUN([]).
 confirm_barrier_test_()                 -> ?RUN([]).
 confirm_barrier_nop_test_()             -> ?RUN([]).
 confirm_barrier_timeout_test_()         -> ?RUN([]).
 confirm_barrier_die_timeout_test_()     -> ?RUN([]).
-default_consumer_test()                 -> ?RUN([]).
+default_consumer_test_()                -> ?RUN([]).
 subscribe_nowait_test_()                -> ?RUN([]).
 
 non_existent_exchange_test_()           -> ?RUN([negative]).
-bogus_rpc_test_()                    -> ?RUN([negative, repeat, {timeout, 60}]).
-hard_error_test_()                   -> ?RUN([negative, repeat, {timeout, 60}]).
+bogus_rpc_test_()                    -> ?RUN([negative, repeat]).
+hard_error_test_()                   -> ?RUN([negative, repeat]).
 non_existent_user_test_()               -> ?RUN([negative]).
 invalid_password_test_()                -> ?RUN([negative]).
 non_existent_vhost_test_()              -> ?RUN([negative]).
@@ -98,7 +99,7 @@ run(TestName, Props) ->
                  true  -> negative_test_util;
                  false -> test_util
              end,
-    {timeout, proplists:get_value(timeout, Props, 10),
+    {timeout, proplists:get_value(timeout, Props, 60),
      fun () ->
              lists:foreach(
                  fun (_) ->
