@@ -11,13 +11,13 @@
 %% The Original Code is RabbitMQ.
 %%
 %% The Initial Developer of the Original Code is VMware, Inc.
-%% Copyright (c) 2011-2012 VMware, Inc.  All rights reserved.
+%% Copyright (c) 2011-2013 VMware, Inc.  All rights reserved.
 %%
 
 -module(test_util).
 
 -include_lib("eunit/include/eunit.hrl").
--include("amqp_client.hrl").
+-include("amqp_client_internal.hrl").
 
 -compile([export_all]).
 
@@ -166,6 +166,11 @@ lifecycle_test() ->
     teardown(Connection, Channel),
     ok.
 
+direct_no_password_test() ->
+    {ok, Connection} = new_connection(just_direct, [{password, none}]),
+    amqp_connection:close(Connection),
+    wait_for_death(Connection).
+
 queue_exchange_binding(Channel, X, Parent, Tag) ->
     receive
         nothing -> ok
@@ -368,11 +373,23 @@ basic_return_test() ->
             ?assertMatch(?NO_ROUTE, ReplyCode),
             #amqp_msg{payload = Payload2} = Content,
             ?assertMatch(Payload, Payload2);
-        WhatsThis ->
-            exit({bad_message, WhatsThis})
+        WhatsThis1 ->
+            exit({bad_message, WhatsThis1})
     after 2000 ->
         exit(no_return_received)
     end,
+    amqp_channel:unregister_return_handler(Channel),
+    Publish = #'basic.publish'{exchange = X, routing_key = Key,
+                               mandatory = true},
+    amqp_channel:call(Channel, Publish, #amqp_msg{payload = Payload}),
+    ok = receive
+             {_BasicReturn = #'basic.return'{}, _Content} ->
+                 unexpected_return;
+             WhatsThis2 ->
+                 exit({bad_message, WhatsThis2})
+         after 2000 ->
+                 ok
+         end,
     teardown(Connection, Channel).
 
 channel_repeat_open_close_test() ->
@@ -990,5 +1007,6 @@ make_direct_params(Props) ->
                   proplists:get_value(Key, Props, Default)
           end,
     #amqp_params_direct{username     = Pgv(username, <<"guest">>),
+                        password     = Pgv(password, <<"guest">>),
                         virtual_host = Pgv(virtual_host, <<"/">>),
                         node         = Pgv(node, node())}.
