@@ -11,7 +11,7 @@
 %% The Original Code is RabbitMQ Federation.
 %%
 %% The Initial Developer of the Original Code is VMware, Inc.
-%% Copyright (c) 2007-2012 VMware, Inc.  All rights reserved.
+%% Copyright (c) 2007-2013 VMware, Inc.  All rights reserved.
 %%
 
 -module(rabbit_federation_db).
@@ -21,13 +21,15 @@
 
 -define(DICT, orddict).
 
+-import(rabbit_federation_util, [name/1]).
+
 -export([get_active_suffix/3, set_active_suffix/3, prune_scratch/2]).
 
 %%----------------------------------------------------------------------------
 
 get_active_suffix(XName, Upstream, Default) ->
-    case rabbit_exchange:lookup(XName) of
-        {ok, #exchange{scratch = Dict}} ->
+    case rabbit_exchange:lookup_scratch(XName, federation) of
+        {ok, Dict} ->
             case ?DICT:find(key(Upstream), Dict) of
                 {ok, Suffix} -> Suffix;
                 error        -> Default
@@ -38,16 +40,19 @@ get_active_suffix(XName, Upstream, Default) ->
 
 set_active_suffix(XName, Upstream, Suffix) ->
     ok = rabbit_exchange:update_scratch(
-           XName, fun(D) -> ?DICT:store(key(Upstream), Suffix, D) end).
+           XName, federation,
+           fun(D) -> ?DICT:store(key(Upstream), Suffix, ensure(D)) end).
 
 prune_scratch(XName, Upstreams) ->
     ok = rabbit_exchange:update_scratch(
-           XName,
-           fun(undefined) -> ?DICT:new();
-              (D)         -> Keys = [key(U) || U <- Upstreams],
-                             ?DICT:filter(
-                                fun(K, _V) -> lists:member(K, Keys) end, D)
+           XName, federation,
+           fun(D) -> Keys = [key(U) || U <- Upstreams],
+                     ?DICT:filter(
+                        fun(K, _V) -> lists:member(K, Keys) end, ensure(D))
            end).
 
-key(#upstream{connection_name = ConnName, exchange = XName}) ->
-    {ConnName, XName}.
+key(#upstream{name = UpstreamName, exchange = X}) ->
+    {UpstreamName, name(X)}.
+
+ensure(undefined) -> ?DICT:new();
+ensure(D)         -> D.
