@@ -11,12 +11,11 @@
 %% The Original Code is RabbitMQ Federation.
 %%
 %% The Initial Developer of the Original Code is VMware, Inc.
-%% Copyright (c) 2007-2012 VMware, Inc.  All rights reserved.
+%% Copyright (c) 2007-2013 VMware, Inc.  All rights reserved.
 %%
 
 -module(rabbit_federation_link).
 
--include_lib("kernel/include/inet.hrl").
 -include_lib("amqp_client/include/amqp_client.hrl").
 -include("rabbit_federation.hrl").
 
@@ -31,10 +30,13 @@
          terminate/2, code_change/3]).
 
 -import(rabbit_misc, [pget/2]).
+-import(rabbit_federation_util, [name/1]).
+-import(rabbit_federation_util, [vhost/1]).
 
 -record(state, {upstream,
                 connection,
                 channel,
+                consumer_tag,
                 queue,
                 internal_exchange,
                 waiting_cmds = gb_trees:empty(),
@@ -45,6 +47,8 @@
                 downstream_exchange,
                 unacked = gb_trees:empty()}).
 
+-define(MAX_CONNECTION_CLOSE_TIMEOUT, 10000).
+
 %%----------------------------------------------------------------------------
 
 %% We start off in a state where we do not connect, since we can first
@@ -69,11 +73,20 @@ start_link(Args) ->
     gen_server2:start_link(?MODULE, Args, [{timeout, infinity}]).
 
 init(Args = {Upstream, XName}) ->
-    rabbit_federation_status:report(Upstream, XName, starting),
-    join(rabbit_federation_exchanges),
-    join({rabbit_federation_exchange, XName}),
-    gen_server2:cast(self(), maybe_go),
-    {ok, {not_started, Args}}.
+    %% If we are starting up due to a policy change then it's possible
+    %% for the exchange to have been deleted before we got here, in which
+    %% case it's possible that delete callback would also have been called
+    %% before we got here. So check if we still exist.
+    case rabbit_exchange:lookup(XName) of
+	{ok, _} ->
+	    rabbit_federation_status:report(Upstream, XName, starting),
+	    join(rabbit_federation_exchanges),
+	    join({rabbit_federation_exchange, XName}),
+	    gen_server2:cast(self(), maybe_go),
+	    {ok, {not_started, Args}};
+	{error, not_found} ->
+	    {stop, gone}
+    end.
 
 handle_call(list_routing_keys, _From, State = #state{bindings = Bindings}) ->
     {reply, lists:sort([K || {K, _} <- dict:fetch_keys(Bindings)]), State};
@@ -122,7 +135,8 @@ handle_info({#'basic.deliver'{routing_key  = Key,
                               delivery_tag = DTag,
                               redelivered  = Redelivered}, Msg},
             State = #state{
-              upstream            = #upstream{max_hops = MaxHops} = Upstream,
+              upstream            = #upstream{max_hops      = MaxHops,
+                                              trust_user_id = Trust} = Upstream,
               downstream_exchange = #resource{name = XNameBin},
               downstream_channel  = DCh,
               unacked             = Unacked}) ->
@@ -133,18 +147,22 @@ handle_info({#'basic.deliver'{routing_key  = Key,
     case rabbit_federation_util:should_forward(Headers0, MaxHops) of
         true  -> {table, Info0} = rabbit_federation_upstream:to_table(Upstream),
                  Info = Info0 ++ [{<<"redelivered">>, bool, Redelivered}],
-                 Headers = rabbit_basic:append_table_header(
+                 Headers = rabbit_basic:prepend_table_header(
                              ?ROUTING_HEADER, Info, Headers0),
                  Seq = amqp_channel:next_publish_seqno(DCh),
                  amqp_channel:cast(DCh, #'basic.publish'{exchange    = XNameBin,
                                                          routing_key = Key},
-                                   update_headers(Headers, Msg)),
+                                   maybe_clear_user_id(
+                                     Trust, update_headers(Headers, Msg))),
                  {noreply, State#state{unacked = gb_trees:insert(Seq, DTag,
                                                                  Unacked)}};
         false -> ack(DTag, false, State), %% Drop it, but acknowledge it!
                  {noreply, State}
     end;
 
+handle_info(#'basic.cancel'{}, State) ->
+    connection_error(local, basic_cancel, State);
+
 %% If the downstream channel shuts down cleanly, we can just ignore it
 %% - we're the same node, we're presumably about to go down too.
 handle_info({'DOWN', _Ref, process, Ch, shutdown},
@@ -171,14 +189,33 @@ handle_info(Msg, State) ->
 terminate(_Reason, {not_started, _}) ->
     ok;
 
-terminate(_Reason, #state{downstream_channel    = DCh,
-                          downstream_connection = DConn,
-                          connection            = Conn,
-                          channel               = Ch}) ->
-    ensure_closed(DConn, DCh),
-    ensure_closed(Conn, Ch),
+terminate(Reason, State = #state{downstream_connection = DConn,
+                                 connection            = Conn}) ->
+    ensure_connection_closed(DConn),
+    ensure_connection_closed(Conn),
+    log_terminate(Reason, State),
     ok.
 
+log_terminate({shutdown, restart}, _State) ->
+    %% We've already logged this before munging the reason
+    ok;
+log_terminate(shutdown, #state{downstream_exchange = XName,
+                               upstream            = Upstream}) ->
+    %% The supervisor is shutting us down; we are probably restarting
+    %% the link because configuration has changed. So try to shut down
+    %% nicely so that we do not cause unacked messages to be
+    %% redelivered.
+    rabbit_log:info("Federation ~s disconnecting from ~s~n",
+                    [rabbit_misc:rs(XName),
+                     rabbit_federation_upstream:to_string(Upstream)]),
+    rabbit_federation_status:remove(Upstream, XName);
+
+log_terminate(Reason, #state{downstream_exchange = XName,
+                             upstream            = Upstream}) ->
+    %% Unexpected death. sasl will log it, but we should update
+    %% rabbit_federation_status.
+    rabbit_federation_status:report(Upstream, XName, clean_reason(Reason)).
+
 code_change(_OldVsn, State, _Extra) ->
     {ok, State}.
 
@@ -251,51 +288,127 @@ open(Params) ->
         E -> E
     end.
 
-add_binding(B = #binding{key = Key, args = Args},
-            State = #state{channel = Ch, internal_exchange = IntXNameBin,
-                           upstream = #upstream{exchange = XNameBin}}) ->
-    case check_add_binding(B, State) of
-        {true,  State1} -> State1;
-        {false, State1} -> amqp_channel:call(
-                             Ch, #'exchange.bind'{
-                               destination = IntXNameBin,
-                               source      = XNameBin,
-                               routing_key = Key,
-                               arguments   = Args}),
-                           State1
+add_binding(B, State) ->
+    binding_op(fun record_binding/2, bind_cmd(bind, B, State), B, State).
+
+remove_binding(B, State) ->
+    binding_op(fun forget_binding/2, bind_cmd(unbind, B, State), B, State).
+
+record_binding(B = #binding{destination = Dest},
+               State = #state{bindings = Bs}) ->
+    {DoIt, Set} = case dict:find(key(B), Bs) of
+                      error       -> {true,  sets:from_list([Dest])};
+                      {ok, Dests} -> {false, sets:add_element(
+                                               Dest, Dests)}
+                  end,
+    {DoIt, State#state{bindings = dict:store(key(B), Set, Bs)}}.
+
+forget_binding(B = #binding{destination = Dest},
+               State = #state{bindings = Bs}) ->
+    Dests = sets:del_element(Dest, dict:fetch(key(B), Bs)),
+    {DoIt, Bs1} = case sets:size(Dests) of
+                      0 -> {true,  dict:erase(key(B), Bs)};
+                      _ -> {false, dict:store(key(B), Dests, Bs)}
+                  end,
+    {DoIt, State#state{bindings = Bs1}}.
+
+binding_op(UpdateFun, Cmd, B = #binding{args = Args},
+           State = #state{channel = Ch}) ->
+    {DoIt, State1} =
+        case rabbit_misc:table_lookup(Args, ?BINDING_HEADER) of
+            undefined  -> UpdateFun(B, State);
+            {array, _} -> {Cmd =/= ignore, State}
+        end,
+    case DoIt of
+        true  -> amqp_channel:call(Ch, Cmd);
+        false -> ok
+    end,
+    State1.
+
+bind_cmd(Type, #binding{key = Key, args = Args},
+         State = #state{internal_exchange = IntXNameBin,
+                        upstream          = Upstream}) ->
+    #upstream{exchange = X} = Upstream,
+    case update_binding(Args, State) of
+        ignore  -> ignore;
+        NewArgs -> bind_cmd0(Type, name(X), IntXNameBin, Key, NewArgs)
     end.
 
-check_add_binding(B = #binding{destination = Dest},
-                  State = #state{bindings = Bs}) ->
-    K = key(B),
-    {Res, Set} = case dict:find(K, Bs) of
-                     {ok, Dests} -> {true,  sets:add_element(Dest, Dests)};
-                     error       -> {false, sets:from_list([Dest])}
-                 end,
-    {Res, State#state{bindings = dict:store(K, Set, Bs)}}.
+bind_cmd0(bind, Source, Destination, RoutingKey, Arguments) ->
+    #'exchange.bind'{source      = Source,
+                     destination = Destination,
+                     routing_key = RoutingKey,
+                     arguments   = Arguments};
 
-remove_binding(B = #binding{key = Key, args = Args},
-               State = #state{channel = Ch,
-                              internal_exchange = IntXNameBin,
-                              upstream = #upstream{exchange = XNameBin}}) ->
-    case check_remove_binding(B, State) of
-        {true,  State1} -> State1;
-        {false, State1} -> amqp_channel:call(
-                             Ch, #'exchange.unbind'{
-                               destination = IntXNameBin,
-                               source      = XNameBin,
-                               routing_key = Key,
-                               arguments   = Args}),
-                           State1
-    end.
+bind_cmd0(unbind, Source, Destination, RoutingKey, Arguments) ->
+    #'exchange.unbind'{source      = Source,
+                       destination = Destination,
+                       routing_key = RoutingKey,
+                       arguments   = Arguments}.
 
-check_remove_binding(B = #binding{destination = Dest},
-                     State = #state{bindings = Bs}) ->
-    K = key(B),
-    Dests = sets:del_element(Dest, dict:fetch(K, Bs)),
-    case sets:size(Dests) of
-        0 -> {false, State#state{bindings = dict:erase(K, Bs)}};
-        _ -> {true,  State#state{bindings = dict:store(K, Dests, Bs)}}
+%% This function adds information about the current node to the
+%% binding arguments, or returns 'ignore' if it determines the binding
+%% should propagate no further. The interesting part is the latter.
+%%
+%% We want bindings to propagate in the same way as messages
+%% w.r.t. max_hops - if we determine that a message can get from node
+%% A to B (assuming bindings are in place) then it follows that a
+%% binding at B should propagate back to A, and no further. There is
+%% no point in propagating bindings past the point where messages
+%% would propagate, and we will lose messages if bindings don't
+%% propagate as far.
+%%
+%% Note that we still want to have limits on how far messages can
+%% propagate: limiting our bindings is not enough, since other
+%% bindings from other nodes can overlap.
+%%
+%% So in short we want bindings to obey max_hops. However, they can't
+%% just obey the max_hops of the current link, since they are
+%% travelling in the opposite direction to messages! Consider the
+%% following federation:
+%%
+%%  A -----------> B -----------> C
+%%     max_hops=1     max_hops=2
+%%
+%% where the arrows indicate message flow. A binding created at C
+%% should propagate to B, then to A, and no further. Therefore every
+%% time we traverse a link, we keep a count of the number of hops that
+%% a message could have made so far to reach this point, and still be
+%% able to propagate. When this number ("hops" below) reaches 0 we
+%% propagate no further.
+%%
+%% hops(link(N)) is given by:
+%%
+%%   min(hops(link(N-1))-1, max_hops(link(N)))
+%%
+%% where link(N) is the link that bindings propagate over after N
+%% steps (e.g. link(1) is CB above, link(2) is BA).
+%%
+%% In other words, we count down to 0 from the link with the most
+%% restrictive max_hops we have yet passed through.
+
+update_binding(Args, #state{downstream_exchange = X,
+                            upstream            = Upstream}) ->
+    #upstream{max_hops = MaxHops} = Upstream,
+    Hops = case rabbit_misc:table_lookup(Args, ?BINDING_HEADER) of
+               undefined    -> MaxHops;
+               {array, All} -> [{table, Prev} | _] = All,
+                               {short, PrevHops} =
+                                   rabbit_misc:table_lookup(Prev, <<"hops">>),
+                               lists:min([PrevHops - 1, MaxHops])
+           end,
+    case Hops of
+        0 -> ignore;
+        _ -> Node = rabbit_federation_util:local_nodename(vhost(X)),
+             ABSuffix = rabbit_federation_db:get_active_suffix(
+                          X, Upstream, <<"A">>),
+             DVHost = vhost(X),
+             DName = name(X),
+             Down = <<Node/binary, ":", DVHost/binary,":", DName/binary, " ",
+                      ABSuffix/binary>>,
+             Info = [{<<"downstream">>, longstr, Down},
+                     {<<"hops">>,       short,   Hops}],
+             rabbit_basic:prepend_table_header(?BINDING_HEADER, Info, Args)
     end.
 
 key(#binding{key = Key, args = Args}) -> {Key, Args}.
@@ -309,7 +422,8 @@ go(S0 = {not_started, {Upstream, DownXName =
     %% us to exit. We can therefore only trap exits when past that
     %% point. Bug 24372 may help us do something nicer.
     process_flag(trap_exit, true),
-    case open_monitor(rabbit_federation_util:local_params(DownVHost)) of
+    case open_monitor(
+           rabbit_federation_util:local_params(Upstream, DownVHost)) of
         {ok, DConn, DCh} ->
             #'confirm.select_ok'{} =
                amqp_channel:call(DCh, #'confirm.select'{}),
@@ -322,6 +436,7 @@ go(S0 = {not_started, {Upstream, DownXName =
                                   {rabbit_exchange:peek_serial(DownXName),
                                    rabbit_binding:list_for_source(DownXName)}
                           end),
+                    true = is_integer(Serial),
                     %% If we are very short lived, Serial can be undefined at
                     %% this point (since the deletion of the X could have
                     %% overtaken the creation of this process). However, this
@@ -351,12 +466,12 @@ go(S0 = {not_started, {Upstream, DownXName =
                     catch exit:E ->
                             %% terminate/2 will not get this, as we
                             %% have not put them in our state yet
-                            ensure_closed(DConn, DCh),
-                            ensure_closed(Conn, Ch),
+                            ensure_connection_closed(DConn),
+                            ensure_connection_closed(Conn),
                             connection_error(remote, E, S0)
                     end;
                 E ->
-                    ensure_closed(DConn, DCh),
+                    ensure_connection_closed(DConn),
                     connection_error(remote, E, S0)
             end;
         E ->
@@ -378,6 +493,14 @@ connection_error(remote, E, State = #state{upstream            = U,
                      rabbit_federation_upstream:to_string(U), E]),
     {stop, {shutdown, restart}, State};
 
+connection_error(local, basic_cancel,
+                 State = #state{upstream            = U,
+                                downstream_exchange = XName}) ->
+    rabbit_federation_status:report(U, XName, {error, basic_cancel}),
+    rabbit_log:info("Federation ~s received 'basic.cancel'~n",
+                    [rabbit_misc:rs(XName)]),
+    {stop, {shutdown, restart}, State};
+
 connection_error(local, E, State = {not_started, {U, XName}}) ->
     rabbit_federation_status:report(U, XName, clean_reason(E)),
     rabbit_log:warning("Federation ~s did not connect locally~n~p~n",
@@ -396,63 +519,73 @@ consume_from_upstream_queue(
   State = #state{upstream            = Upstream,
                  channel             = Ch,
                  downstream_exchange = DownXName}) ->
-    #upstream{exchange       = XNameBin,
+    #upstream{exchange       = X,
               prefetch_count = Prefetch,
               expires        = Expiry,
               message_ttl    = TTL,
               ha_policy      = HA,
-              params         = #amqp_params_network{virtual_host = VHost}}
+              params         = Params}
         = Upstream,
-    Q = upstream_queue_name(XNameBin, VHost, DownXName),
-    %% TODO it would be nice to just pass through args, but let's do that as
-    %% part of bug 23908.
-    ExpiryArg = case Expiry of
-                    none -> [];
-                    _    -> [{<<"x-expires">>, long, Expiry}]
-                end,
-    TTLArg = case TTL of
-                 none -> [];
-                 _    -> [{<<"x-message-ttl">>, long, TTL}]
-             end,
-    HAArg = case HA of
-                none -> [];
-                _    -> [{<<"x-ha-policy">>, longstr, list_to_binary(HA)}]
-            end,
-    Args = ExpiryArg ++ TTLArg ++ HAArg,
+    Q = upstream_queue_name(name(X), vhost(Params), DownXName),
+    Args = [Arg || {_K, _T, V} = Arg <- [{<<"x-expires">>,     long,    Expiry},
+                                         {<<"x-message-ttl">>, long,    TTL},
+                                         {<<"x-ha-policy">>,   longstr, HA}],
+                   V =/= none],
     amqp_channel:call(Ch, #'queue.declare'{queue     = Q,
                                            durable   = true,
                                            arguments = Args}),
     amqp_channel:call(Ch, #'basic.qos'{prefetch_count = Prefetch}),
-    #'basic.consume_ok'{} =
+    #'basic.consume_ok'{consumer_tag = CTag} =
         amqp_channel:subscribe(Ch, #'basic.consume'{queue  = Q,
                                                     no_ack = false}, self()),
-    State#state{queue = Q}.
+    State#state{consumer_tag = CTag,
+                queue        = Q}.
 
 ensure_upstream_bindings(State = #state{upstream            = Upstream,
                                         connection          = Conn,
                                         channel             = Ch,
                                         downstream_exchange = DownXName,
                                         queue               = Q}, Bindings) ->
-    #upstream{exchange = XNameBin,
-              params   = #amqp_params_network{virtual_host = VHost}} = Upstream,
+    #upstream{exchange = X, params = Params} = Upstream,
     OldSuffix = rabbit_federation_db:get_active_suffix(
                   DownXName, Upstream, <<"A">>),
     Suffix = case OldSuffix of
                  <<"A">> -> <<"B">>;
                  <<"B">> -> <<"A">>
              end,
-    IntXNameBin = upstream_exchange_name(XNameBin, VHost, DownXName, Suffix),
-    ensure_upstream_exchange(IntXNameBin, State),
+    IntXNameBin = upstream_exchange_name(name(X), vhost(Params),
+                                         DownXName, Suffix),
+    ensure_upstream_exchange(State),
+    ensure_internal_exchange(IntXNameBin, State),
     amqp_channel:call(Ch, #'queue.bind'{exchange = IntXNameBin, queue = Q}),
     State1 = State#state{internal_exchange = IntXNameBin},
-    State2 = lists:foldl(fun add_binding/2, State1, Bindings),
     rabbit_federation_db:set_active_suffix(DownXName, Upstream, Suffix),
+    State2 = lists:foldl(fun add_binding/2, State1, Bindings),
     OldIntXNameBin = upstream_exchange_name(
-                       XNameBin, VHost, DownXName, OldSuffix),
+                       name(X), vhost(Params), DownXName, OldSuffix),
     delete_upstream_exchange(Conn, OldIntXNameBin),
     State2.
 
-ensure_upstream_exchange(IntXNameBin,
+ensure_upstream_exchange(#state{upstream   = #upstream{exchange = X},
+                                connection = Conn,
+                                channel    = Ch}) ->
+    #exchange{type        = Type,
+              durable     = Durable,
+              auto_delete = AutoDelete,
+              internal    = Internal,
+              arguments   = Arguments} = X,
+    Decl = #'exchange.declare'{exchange    = name(X),
+                               type        = list_to_binary(atom_to_list(Type)),
+                               durable     = Durable,
+                               auto_delete = AutoDelete,
+                               internal    = Internal,
+                               arguments   = Arguments},
+    disposable_channel_call(Conn, Decl#'exchange.declare'{passive = true},
+                           fun(?NOT_FOUND, _Text) ->
+                                   amqp_channel:call(Ch, Decl)
+                           end).
+
+ensure_internal_exchange(IntXNameBin,
                          #state{upstream   = #upstream{max_hops = MaxHops,
                                                        params   = Params},
                                 connection = Conn,
@@ -471,7 +604,7 @@ ensure_upstream_exchange(IntXNameBin,
 
 upstream_queue_name(XNameBin, VHost, #resource{name         = DownXNameBin,
                                                virtual_host = DownVHost}) ->
-    Node = local_nodename(),
+    Node = rabbit_federation_util:local_nodename(DownVHost),
     DownPart = case DownVHost of
                    VHost -> case DownXNameBin of
                                 XNameBin -> <<"">>;
@@ -486,17 +619,6 @@ upstream_exchange_name(XNameBin, VHost, DownXName, Suffix) ->
     Name = upstream_queue_name(XNameBin, VHost, DownXName),
     <<Name/binary, " ", Suffix/binary>>.
 
-local_nodename() ->
-    {ok, Explicit} = application:get_env(rabbitmq_federation, local_nodename),
-    case Explicit of
-        automatic -> {ID, _} = rabbit_nodes:parts(node()),
-                     {ok, Host} = inet:gethostname(),
-                     {ok, #hostent{h_name = FQDN}} = inet:gethostbyname(Host),
-                     list_to_binary(atom_to_list(
-                                      rabbit_nodes:make({ID, FQDN})));
-        _         -> list_to_binary(Explicit)
-    end.
-
 delete_upstream_exchange(Conn, XNameBin) ->
     disposable_channel_call(Conn, #'exchange.delete'{exchange = XNameBin}).
 
@@ -509,8 +631,9 @@ disposable_channel_call(Conn, Method, ErrFun) ->
         amqp_channel:call(Ch, Method)
     catch exit:{{shutdown, {server_initiated_close, Code, Text}}, _} ->
             ErrFun(Code, Text)
-    end,
-    ensure_closed(Ch).
+    after
+        ensure_channel_closed(Ch)
+    end.
 
 disposable_connection_call(Params, Method, ErrFun) ->
     case open(Params) of
@@ -520,18 +643,17 @@ disposable_connection_call(Params, Method, ErrFun) ->
             catch exit:{{shutdown, {connection_closing,
                                     {server_initiated_close, Code, Txt}}}, _} ->
                     ErrFun(Code, Txt)
-            end,
-            ensure_closed(Conn, Ch);
+            after
+                ensure_connection_closed(Conn)
+            end;
         E ->
             E
     end.
 
-ensure_closed(Conn, Ch) ->
-    ensure_closed(Ch),
-    catch amqp_connection:close(Conn).
+ensure_channel_closed(Ch) -> catch amqp_channel:close(Ch).
 
-ensure_closed(Ch) ->
-    catch amqp_channel:close(Ch).
+ensure_connection_closed(Conn) ->
+    catch amqp_connection:close(Conn, ?MAX_CONNECTION_CLOSE_TIMEOUT).
 
 ack(Tag, Multiple, #state{channel = Ch}) ->
     amqp_channel:cast(Ch, #'basic.ack'{delivery_tag = Tag,
@@ -552,5 +674,10 @@ remove_delivery_tags(Seq, true, Unacked) ->
 extract_headers(#amqp_msg{props = #'P_basic'{headers = Headers}}) ->
     Headers.
 
+maybe_clear_user_id(false, Msg = #amqp_msg{props = Props}) ->
+    Msg#amqp_msg{props = Props#'P_basic'{user_id = undefined}};
+maybe_clear_user_id(true, Msg) ->
+    Msg.
+
 update_headers(Headers, Msg = #amqp_msg{props = Props}) ->
     Msg#amqp_msg{props = Props#'P_basic'{headers = Headers}}.
