@@ -11,7 +11,7 @@
 %% The Original Code is RabbitMQ.
 %%
 %% The Initial Developer of the Original Code is VMware, Inc.
-%% Copyright (c) 2007-2012 VMware, Inc.  All rights reserved.
+%% Copyright (c) 2007-2013 VMware, Inc.  All rights reserved.
 %%
 
 -module(rabbit_jsonrpc_channel).
@@ -255,9 +255,9 @@ init([Oid, [Username, Password, SessionTimeout0, VHostPath0]]) ->
     rabbit_log:info("HTTP Channel started, timeout ~p~n", [SessionTimeout]),
     SessionTimeoutMs = SessionTimeout * 1000,
 
-    AdapterInfo = #adapter_info{protocol = {'JSON-RPC', "1.0"}},
-    {ok, _} = rabbit_access_control:check_user_pass_login(Username, Password),
+    AdapterInfo = #amqp_adapter_info{protocol = {'JSON-RPC', "1.0"}},
     Params = #amqp_params_direct{username     = Username,
+                                 password     = Password,
                                  virtual_host = VHostPath,
                                  adapter_info = AdapterInfo},
     {ok, Conn} = amqp_connection:start(Params),
