@@ -11,14 +11,14 @@
 %%   The Original Code is RabbitMQ Management Console.
 %%
 %%   The Initial Developer of the Original Code is VMware, Inc.
-%%   Copyright (c) 2010-2012 VMware, Inc.  All rights reserved.
+%%   Copyright (c) 2010-2013 VMware, Inc.  All rights reserved.
 %%
 
 -module(rabbit_mgmt_db_handler).
 
 -behaviour(gen_event).
 
--export([add_handler/0]).
+-export([add_handler/0, gc/0]).
 
 -export([init/1, handle_call/2, handle_event/2, handle_info/2,
          terminate/2, code_change/3]).
@@ -29,6 +29,9 @@ add_handler() ->
     ensure_statistics_enabled(),
     gen_event:add_sup_handler(rabbit_event, ?MODULE, []).
 
+gc() ->
+    erlang:garbage_collect(whereis(rabbit_event)).
+
 %%----------------------------------------------------------------------------
 
 ensure_statistics_enabled() ->
