@@ -11,7 +11,7 @@
 %%   The Original Code is RabbitMQ Management Console.
 %%
 %%   The Initial Developer of the Original Code is VMware, Inc.
-%%   Copyright (c) 2010-2012 VMware, Inc.  All rights reserved.
+%%   Copyright (c) 2010-2013 VMware, Inc.  All rights reserved.
 %%
 
 -module(rabbit_mgmt_external_stats).
@@ -22,38 +22,27 @@
 -export([init/1, handle_call/3, handle_cast/2, handle_info/2, terminate/2,
          code_change/3]).
 
--export([info/1, info/2]).
-
 -export([list_registry_plugins/1]).
 
 -include_lib("rabbit_common/include/rabbit.hrl").
 
 -define(REFRESH_RATIO, 5000).
--define(KEYS, [os_pid, mem_ets, mem_binary, mem_proc, mem_proc_used,
-               mem_atom, mem_atom_used, mem_code, fd_used, fd_total,
+-define(KEYS, [name, partitions, os_pid, fd_used, fd_total,
                sockets_used, sockets_total, mem_used, mem_limit, mem_alarm,
                disk_free_limit, disk_free, disk_free_alarm,
                proc_used, proc_total, statistics_level,
-               erlang_version, uptime, run_queue, processors, exchange_types,
-               auth_mechanisms, applications]).
+               uptime, run_queue, processors, exchange_types,
+               auth_mechanisms, applications, contexts]).
 
 %%--------------------------------------------------------------------
 
--record(state, {time_ms, fd_used, fd_total}).
+-record(state, {fd_total}).
 
 %%--------------------------------------------------------------------
 
 start_link() ->
     gen_server:start_link({local, ?MODULE}, ?MODULE, [], []).
 
-info(Node) ->
-    info(Node, ?KEYS).
-
-info(Node, Keys) ->
-    rabbit_misc:with_exit_handler(
-      fun() -> [{external_stats_not_running, true}] end,
-      fun() -> gen_server2:call({?MODULE, Node}, {info, Keys}, infinity) end).
-
 %%--------------------------------------------------------------------
 
 get_used_fd_lsof() ->
@@ -153,31 +142,27 @@ get_disk_free() -> ?SAFE_CALL(rabbit_disk_monitor:get_disk_free(),
 
 infos(Items, State) -> [{Item, i(Item, State)} || Item <- Items].
 
-i(fd_used,  #state{fd_used  = FdUsed})  -> FdUsed;
+i(name,            _State) -> node();
+i(partitions,      _State) -> {_Node, Parts} = rabbit_node_monitor:partitions(),
+                              Parts;
+i(fd_used,         _State) -> get_used_fd();
 i(fd_total, #state{fd_total = FdTotal}) -> FdTotal;
 i(sockets_used,    _State) ->
     proplists:get_value(sockets_used, file_handle_cache:info([sockets_used]));
 i(sockets_total,   _State) ->
     proplists:get_value(sockets_limit, file_handle_cache:info([sockets_limit]));
 i(os_pid,          _State) -> list_to_binary(os:getpid());
-i(mem_ets,         _State) -> erlang:memory(ets);
-i(mem_binary,      _State) -> erlang:memory(binary);
-i(mem_proc,        _State) -> erlang:memory(processes);
-i(mem_proc_used,   _State) -> erlang:memory(processes_used);
-i(mem_atom,        _State) -> erlang:memory(atom);
-i(mem_atom_used,   _State) -> erlang:memory(atom_used);
-i(mem_code,        _State) -> erlang:memory(code);
 i(mem_used,        _State) -> erlang:memory(total);
 i(mem_limit,       _State) -> vm_memory_monitor:get_memory_limit();
 i(mem_alarm,       _State) -> resource_alarm_set(memory);
 i(proc_used,       _State) -> erlang:system_info(process_count);
 i(proc_total,      _State) -> erlang:system_info(process_limit);
-i(erlang_version,  _State) -> list_to_binary(erlang:system_info(otp_release));
 i(run_queue,       _State) -> erlang:statistics(run_queue);
 i(processors,      _State) -> erlang:system_info(logical_processors);
 i(disk_free_limit, _State) -> get_disk_free_limit();
 i(disk_free,       _State) -> get_disk_free();
 i(disk_free_alarm, _State) -> resource_alarm_set(disk);
+i(contexts,        _State) -> rabbit_mochiweb_contexts();
 i(uptime, _State) ->
     {Total, _} = erlang:statistics(wall_clock),
     Total;
@@ -197,7 +182,7 @@ i(applications, _State) ->
 
 resource_alarm_set(Source) ->
     lists:member({{resource_limit, Source, node()},[]},
-                 alarm_handler:get_alarms()).
+                 rabbit_alarm:get_alarms()).
 
 list_registry_plugins(Type) ->
     list_registry_plugins(Type, fun(_) -> true end).
@@ -216,18 +201,48 @@ format_application({Application, Description, Version}) ->
 
 %%--------------------------------------------------------------------
 
+%% This is slightly icky in that we introduce knowledge of
+%% rabbit_mochiweb, which is not a dependency. But the last thing I
+%% want to do is create a rabbitmq_mochiweb_management_agent plugin.
+rabbit_mochiweb_contexts() ->
+    [format_context(C) || C <- rabbit_mochiweb_registry_list_all()].
+
+%% For similar reasons we don't declare a dependency on
+%% rabbitmq_mochiweb - so at startup there's no guarantee it will be
+%% running. So we have to catch this noproc.
+rabbit_mochiweb_registry_list_all() ->
+    case code:is_loaded(rabbit_mochiweb_registry) of
+        false -> [];
+        _     -> try
+                     apply0(rabbit_mochiweb_registry, list_all, [])
+                 catch exit:{noproc, _} ->
+                         []
+                 end
+    end.
+
+%% Fool xref. Simply using apply(M, F, A) with constants is not enough.
+apply0(M, F, A) -> apply(M, F, A).
+
+format_context({Path, Description, Rest}) ->
+    [{description, list_to_binary(Description)},
+     {path,        list_to_binary("/" ++ Path)} |
+     format_mochiweb_option_list(Rest)].
+
+format_mochiweb_option_list(C) ->
+    [{K, format_mochiweb_option(K, V)} || {K, V} <- C].
+
+format_mochiweb_option(ssl_opts, V) ->
+    format_mochiweb_option_list(V);
+format_mochiweb_option(_K, V) when is_list(V) ->
+    list_to_binary(V);
+format_mochiweb_option(_K, V) ->
+    V.
+
+%%--------------------------------------------------------------------
+
 init([]) ->
-    State = #state{fd_total   = file_handle_cache:ulimit()},
-    {ok, internal_update(State)}.
-
-
-handle_call({info, Items}, _From, State0) ->
-    State = case (rabbit_misc:now_ms() - State0#state.time_ms >
-                      ?REFRESH_RATIO) of
-                true  -> internal_update(State0);
-                false -> State0
-            end,
-    {reply, infos(Items, State), State};
+    State = #state{fd_total = file_handle_cache:ulimit()},
+    {ok, emit_update(State)}.
 
 handle_call(_Req, _From, State) ->
     {reply, unknown_request, State}.
@@ -235,6 +250,9 @@ handle_call(_Req, _From, State) ->
 handle_cast(_C, State) ->
     {noreply, State}.
 
+handle_info(emit_update, State) ->
+    {noreply, emit_update(State)};
+
 handle_info(_I, State) ->
     {noreply, State}.
 
@@ -244,6 +262,7 @@ code_change(_, State, _) -> {ok, State}.
 
 %%--------------------------------------------------------------------
 
-internal_update(State) ->
-    State#state{time_ms   = rabbit_misc:now_ms(),
-                fd_used   = get_used_fd()}.
+emit_update(State) ->
+    rabbit_event:notify(node_stats, infos(?KEYS, State)),
+    erlang:send_after(?REFRESH_RATIO, self(), emit_update),
+    State.
