@@ -153,6 +153,28 @@ dispatcher_add(function(sammy) {
                 update();
             return false;
         });
+    path('#/policies', {'policies': '/policies',
+                        'vhosts':   '/vhosts'}, 'policies');
+    sammy.get('#/policies/:vhost/:id', function() {
+            render({'policy': '/policies/' + esc(this.params['vhost'])
+                        + '/' + esc(this.params['id'])},
+                'policy', '#/policies');
+        });
+    sammy.put('#/policies', function() {
+            put_policy(this, ['name', 'pattern', 'policy'], ['priority'], []);
+            return false;
+        });
+    sammy.del('#/policies', function() {
+            if (sync_delete(this, '/policies/:vhost/:name'))
+                go_to('#/policies');
+            return false;
+        });
+
+    sammy.put('#/logout', function() {
+            document.cookie = 'auth=; expires=Thu, 01 Jan 1970 00:00:00 GMT';
+            location.reload();
+        });
+
     sammy.get('#/import-succeeded', function() {
             render({}, 'import-succeeded', '#/overview');
         });
