@@ -14,7 +14,9 @@ function map(list) {
 }
 
 // Which queries need to have the current vhost appended (if there is one)?
-var VHOST_QUERIES = map(['/queues', '/exchanges']);
+var VHOST_QUERIES = [new RegExp('^/queues$'),
+                     new RegExp('^/exchanges$'),
+                     new RegExp('^/parameters/[^/]*$')];
 
 // Which queries need to have the current sort appended (if there is one)?
 var SORT_QUERIES  = map(['/connections', '/channels', '/vhosts', '/users',
@@ -24,7 +26,6 @@ var SORT_QUERIES  = map(['/connections', '/channels', '/vhosts', '/users',
 var KNOWN_ARGS = {'alternate-exchange':        {'short': 'AE',  'type': 'string'},
                   'x-message-ttl':             {'short': 'TTL', 'type': 'int'},
                   'x-expires':                 {'short': 'Exp', 'type': 'int'},
-                  'x-ha-policy':               {'short': 'HA',  'type': 'string'},
                   'x-dead-letter-exchange':    {'short': 'DLX', 'type': 'string'},
                   'x-dead-letter-routing-key': {'short': 'DLK', 'type': 'string'}};
 
@@ -38,6 +39,15 @@ var ALL_ARGS = {};
 for (var k in KNOWN_ARGS)    ALL_ARGS[k] = KNOWN_ARGS[k];
 for (var k in IMPLICIT_ARGS) ALL_ARGS[k] = IMPLICIT_ARGS[k];
 
+var NAVIGATION = {'Overview':    ['#/',                            false],
+                  'Connections': ['#/connections',                 false],
+                  'Channels':    ['#/channels',                    false],
+                  'Exchanges':   ['#/exchanges',                   false],
+                  'Queues':      ['#/queues',                      false],
+                  'Admin':       [{'Users':         ['#/users',    true],
+                                   'Virtual Hosts': ['#/vhosts',   true],
+                                   'Policies':      ['#/policies', true]}, true]
+                 };
 
 ///////////////////////////////////////////////////////////////////////////
 //                                                                       //
@@ -46,11 +56,12 @@ for (var k in IMPLICIT_ARGS) ALL_ARGS[k] = IMPLICIT_ARGS[k];
 ///////////////////////////////////////////////////////////////////////////
 
 // All these are to do with hiding UI elements if
-var statistics_level;   // ...there are no fine stats
-var user_administrator; // ...user is not an admin
-var user_monitor;       // ...user cannot monitor
-var nodes_interesting;  // ...we are not in a cluster
-var vhosts_interesting; // ...there is only one vhost
+var statistics_level;            // ...there are no fine stats
+var user_administrator;          // ...user is not an admin
+var user_monitor;                // ...user cannot monitor
+var nodes_interesting;           // ...we are not in a cluster
+var vhosts_interesting;          // ...there is only one vhost
+var rabbit_versions_interesting; // ...all cluster nodes run the same version
 
 // Extensions write to this, the dispatcher maker reads it
 var dispatcher_modules = [];
@@ -61,20 +72,37 @@ var extension_count;
 // The dispatcher needs access to the Sammy app
 var app;
 
+// Used for the new exchange form, and to display broken exchange types
 var exchange_types;
 
 // Set up the above vars
-function setup_global_vars() {
+function setup_global_vars(user) {
     var overview = JSON.parse(sync_get('/overview'));
     statistics_level = overview.statistics_level;
-    var user = JSON.parse(sync_get('/whoami'));
-    replace_content('login', '<p>User: <b>' + user.name + '</b></p>');
+    replace_content('login-details',
+                    '<p>User: <b>' + user.name + '</b></p>' +
+                    '<p>RabbitMQ ' + overview.rabbitmq_version +
+                    ', Erlang ' + overview.erlang_version + '</p>');
     var tags = user.tags.split(",");
     user_administrator = jQuery.inArray("administrator", tags) != -1;
     user_monitor = user_administrator ||
         jQuery.inArray("monitoring", tags) != -1;
-    nodes_interesting = user_monitor &&
-        JSON.parse(sync_get('/nodes')).length > 1;
+    nodes_interesting = false;
+    rabbit_versions_interesting = false;
+    if (user_monitor) {
+        var nodes = JSON.parse(sync_get('/nodes'));
+        if (nodes.length > 1) {
+            nodes_interesting = true;
+            var v = '';
+            for (var i = 0; i < nodes.length; i++) {
+                var v1 = fmt_rabbit_version(nodes[i].applications);
+                if (v1 != 'unknown') {
+                    if (v != '' && v != v1) rabbit_versions_interesting = true;
+                    v = v1;
+                }
+            }
+        }
+    }
     vhosts_interesting = JSON.parse(sync_get('/vhosts')).length > 1;
     current_vhost = get_pref('vhost');
     exchange_types = overview.exchange_types;
