@@ -6,28 +6,28 @@ HELP = {
       'If yes, clients cannot publish to this exchange directly. It can only be used with exchange to exchange bindings.',
 
     'exchange-alternate':
-      'If messages to this exchange cannot otherwise be routed, send them to the alternate exchange named here.<br/>(Sets the "alternate-exchange" argument.)',
+      'If messages to this exchange cannot otherwise be routed, send them to the alternate exchange named here.<br/>(Sets the "<a target="_blank" href="http://rabbitmq.com/ae.html">alternate-exchange</a>" argument.)',
 
     'queue-message-ttl':
-    'How long a message published to a queue can live before it is discarded (milliseconds).<br/>(Sets the "x-message-ttl" argument.)',
+    'How long a message published to a queue can live before it is discarded (milliseconds).<br/>(Sets the "<a target="_blank" href="http://rabbitmq.com/ttl.html#per-queue-message-ttl">x-message-ttl</a>" argument.)',
 
     'queue-expires':
-      'How long a queue can be unused for before it is automatically deleted (milliseconds).<br/>(Sets the "x-expires" argument.)',
+      'How long a queue can be unused for before it is automatically deleted (milliseconds).<br/>(Sets the "<a target="_blank" href="http://rabbitmq.com/ttl.html#queue-ttl">x-expires</a>" argument.)',
 
     'queue-auto-delete':
       'If yes, the queue will delete itself after at least one consumer has connected, and then all consumers have disconnected.',
 
     'queue-dead-letter-exchange':
-      'Optional name of an exchange to which messages will be republished if they are rejected or expire.<br/>(Sets the "x-dead-letter-exchange" argument.)',
+      'Optional name of an exchange to which messages will be republished if they are rejected or expire.<br/>(Sets the "<a target="_blank" href="http://rabbitmq.com/dlx.html">x-dead-letter-exchange</a>" argument.)',
 
     'queue-dead-letter-routing-key':
-      'Optional replacement routing key to use when a message is dead-lettered. If this is not set, the message\'s original routing key will be used.<br/>(Sets the "x-dead-letter-routing-key" argument.)',
+      'Optional replacement routing key to use when a message is dead-lettered. If this is not set, the message\'s original routing key will be used.<br/>(Sets the "<a target="_blank" href="http://rabbitmq.com/dlx.html">x-dead-letter-routing-key</a>" argument.)',
 
     'internal-users-only':
       'Only users within the internal RabbitMQ database are shown here. Other users (e.g. those authenticated over LDAP) will not appear.',
 
     'export-definitions':
-    'The definitions consist of users, virtual hosts, permissions, exchanges, queues and bindings. They do not include the contents of queues. Exclusive queues will not be exported.',
+    'The definitions consist of users, virtual hosts, permissions, parameters, exchanges, queues and bindings. They do not include the contents of queues. Exclusive queues will not be exported.',
 
     'import-definitions':
       'The definitions that are imported will be merged with the current definitions. If an error occurs during import, any changes made will not be rolled back.',
@@ -41,9 +41,9 @@ HELP = {
     'channel-mode':
       'Channel guarantee mode. Can be one of the following, or neither:<br/> \
       <dl> \
-        <dt><acronym title="Confirm">C</acronym> &ndash; confirm</dt> \
+        <dt><acronym title="Confirm">C</acronym> &ndash; <a target="_blank" href="http://www.rabbitmq.com/confirms.html">confirm</a></dt> \
         <dd>Channel will send streaming publish confirmations.</dd> \
-        <dt><acronym title="Transactional">T</acronym> &ndash; transactional</dt> \
+        <dt><acronym title="Transactional">T</acronym> &ndash; <a target="_blank" href="http://www.rabbitmq.com/amqp-0-9-1-reference.html#class.tx">transactional</a></dt> \
         <dd>Channel is transactional.</dd> \
       <dl>',
 
@@ -58,7 +58,7 @@ HELP = {
       the limit on Windows, set the ERL_MAX_PORTS environment variable</p> \
       <p>To report used file handles on Windows, handle.exe from \
       sysinternals must be installed in your path. You can download it \
-      <a href="http://technet.microsoft.com/en-us/sysinternals/bb896655">here</a>.</p>',
+      <a target="_blank" href="http://technet.microsoft.com/en-us/sysinternals/bb896655">here</a>.</p>',
 
     'socket-descriptors':
       'The network sockets count and limit managed by RabbitMQ.<br/> \
@@ -66,12 +66,16 @@ HELP = {
       network connections.',
 
     'memory-alarm':
-      'The memory alarm for this node has gone off. It will block \
+      '<p>The <a target="_blank" href="http://www.rabbitmq.com/memory.html#memsup">memory \
+      alarm</a> for this node has gone off. It will block \
       incoming network traffic until the memory usage drops below \
-      the watermark.',
+      the watermark.</p>\
+      <p>Note that the pale line in this case indicates the high watermark \
+      in relation to how much memory is used in total. </p>',
 
     'disk-free-alarm':
-      'The disk free space alarm for this node has gone off. It will block \
+      'The <a target="_blank" href="http://www.rabbitmq.com/memory.html#diskfreesup">disk \
+      free space alarm</a> for this node has gone off. It will block \
       incoming network traffic until the amount of free space exceeds \
       the limit.',
 
@@ -118,8 +122,9 @@ HELP = {
      </dl>',
 
     'user-tags':
-      'Comma-separated list of tags to apply to the user. Currently supported \
-       by the management plugin: \
+      'Comma-separated list of tags to apply to the user. Currently \
+       <a target="_blank" href="http://www.rabbitmq.com/management.html#permissions">supported \
+       by the management plugin</a>: \
       <dl> \
         <dt>management</dt> \
         <dd> \
@@ -135,7 +140,11 @@ HELP = {
           User can do everything monitoring can do, manage users, \
           vhosts and permissions, and close other user\'s connections. \
         </dd> \
-      </dl>',
+      </dl> \
+      <p> \
+        Note that you can set any tag here; the links for the above three \
+        tags are just for convenience. \
+      </p>',
 
     'queued-messages':
     'Total messages in all queues:\
@@ -169,13 +178,38 @@ HELP = {
         <dd>Rate at which messages are being acknowledged.</dd>\
         <dt>Redelivered</dt>\
         <dd>Rate at which messages with the \'redelivered\' flag set are being delivered. Note that these messages will <b>also</b> be counted in one of the delivery rates above.</dd>\
-        <dt>Return (mandatory)</dt>\
-        <dd>Rate at which basic.return is sent to publishers for unroutable  messages published with the \'mandatory\' flag set.</dd>\
-        <dt>Return (immediate)</dt>\
-        <dd>Rate at which basic.return is sent to publishers for undeliverable  messages published with the \'immediate\' flag set.</dd>\
+        <dt>Return</dt>\
+        <dd>Rate at which basic.return is sent to publishers for unroutable messages published with the \'mandatory\' flag set.</dd>\
       </dl>',
 
-    'disk-monitoring-no-watermark' : 'There is no disk space low watermark set. RabbitMQ will not take any action to avoid running out of disk space.',
+    'disk-monitoring-no-watermark' : 'There is no <a target="_blank" href="http://www.rabbitmq.com/memory.html#diskfreesup">disk space low watermark</a> set. RabbitMQ will not take any action to avoid running out of disk space.',
+
+    'resource-counts' : 'Shows total number of objects for all virtual hosts the current user has access to.',
+
+    'active-consumers' : '<p>An active consumer is one which could immediately receive any messages sent to the queue - i.e. it is not limited by its prefetch count, TCP congestion, flow control, or because it has issued channel.flow. Therefore at least one of Ready Messages and Active Consumers must always be zero.</p><p>Note that this value is an instantaneous snapshot - when consumers are restricted by their prefetch count they may only appear to be active for small fractions of a second until more messages are sent out.</p>',
+
+    'memory-use' : '<p>Note that the memory details shown here are only updated on request - they could be too expensive to calculate every few seconds on a busy server.</p><p><a target="_blank" href="http://www.rabbitmq.com/memory-use.html">Read more</a> on memory use.</p>',
+
+    'policy-definitions' : '<dl>\
+<dt><code>ha-mode</code></dt>\
+  <dd>\
+    One of <code>all</code>, <code>exactly</code>\
+    or <code>nodes</code> (the latter currently not supported by\
+    web UI).\
+  </dd>\
+  <dt><code>ha-params</code></dt>\
+  <dd>\
+    Absent if <code>ha-mode</code> is <code>all</code>, a number\
+    if <code>ha-mode</code> is <code>exactly</code>, or an array\
+    of strings if <code>ha-mode</code> is <code>nodes</code>.\
+  </dd>\
+  <dt><code>federation-upstream-set</code></dt>\
+  <dd>\
+    A string; only if the federation plugin is enabled.\
+  </dd>\
+</dl>',
+
+    'handle-exe' : 'In order to monitor the number of file descriptors in use on Windows, RabbitMQ needs the <a href="http://technet.microsoft.com/en-us/sysinternals/bb896655" target="_blank">handle.exe command line tool from Microsoft</a>. Download it and place it in the path (e.g. in C:\Windows).',
 
     'foo': 'foo' // No comma.
 };
