@@ -11,7 +11,7 @@
 %%   The Original Code is RabbitMQ Management Plugin.
 %%
 %%   The Initial Developer of the Original Code is VMware, Inc.
-%%   Copyright (c) 2010-2012 VMware, Inc.  All rights reserved.
+%%   Copyright (c) 2010-2013 VMware, Inc.  All rights reserved.
 %%
 
 -module(rabbit_mgmt_app).
@@ -23,71 +23,59 @@
 -include_lib("amqp_client/include/amqp_client.hrl").
 
 -define(CONTEXT, rabbit_mgmt).
+-define(CONTEXT_REDIRECT, rabbit_mgmt_redirect).
 -define(STATIC_PATH, "priv/www").
 
--ifdef(trace).
--define(SETUP_WM_TRACE, true).
--else.
--define(SETUP_WM_TRACE, false).
--endif.
-
 start(_Type, _StartArgs) ->
+    {ok, Listener} = application:get_env(rabbitmq_management, listener),
     setup_wm_logging(),
-    register_context(),
-    case ?SETUP_WM_TRACE of
-        true -> setup_wm_trace_app();
-        _    -> ok
-    end,
-    log_startup(),
+    register_context(Listener),
+    log_startup(Listener),
     rabbit_mgmt_sup:start_link().
 
 stop(_State) ->
     unregister_context(),
     ok.
 
-register_context() ->
+register_context(Listener) ->
+    if_redirect(
+      fun () ->
+              rabbit_mochiweb:register_port_redirect(
+                ?CONTEXT_REDIRECT, [{port,          55672},
+                                    {ignore_in_use, true}], "", port(Listener))
+      end),
     rabbit_mochiweb:register_context_handler(
-      ?CONTEXT, "", make_loop(), "RabbitMQ Management").
+      ?CONTEXT, Listener, "", make_loop(), "RabbitMQ Management").
 
 unregister_context() ->
+    if_redirect(
+      fun () -> rabbit_mochiweb:unregister_context(?CONTEXT_REDIRECT) end),
     rabbit_mochiweb:unregister_context(?CONTEXT).
 
+if_redirect(Thunk) ->
+    {ok, Redir} = application:get_env(rabbitmq_management, redirect_old_port),
+    case Redir of
+        true  -> Thunk();
+        false -> ok
+    end.
+
 make_loop() ->
     Dispatch = rabbit_mgmt_dispatcher:build_dispatcher(),
     WMLoop = rabbit_webmachine:makeloop(Dispatch),
     LocalPaths = [filename:join(module_path(M), ?STATIC_PATH) ||
                      M <- rabbit_mgmt_dispatcher:modules()],
-    fun(PL, Req) ->
-            Unauthorized = {401, [{"WWW-Authenticate", ?AUTH_REALM}], ""},
-            Auth = Req:get_header_value("authorization"),
-            case rabbit_mochiweb_util:parse_auth_header(Auth) of
-                [Username, Password] ->
-                    case rabbit_access_control:check_user_pass_login(
-                           Username, Password) of
-                        {ok, _} -> respond(Req, LocalPaths, PL, WMLoop);
-                        _       -> Req:respond(Unauthorized)
-                    end;
-                _ ->
-                    Req:respond(Unauthorized)
-            end
-
-    end.
+    fun(Req) -> respond(Req, LocalPaths, WMLoop) end.
 
 module_path(Module) ->
     {file, Here} = code:is_loaded(Module),
     filename:dirname(filename:dirname(Here)).
 
-respond(Req, LocalPaths, PL = {Prefix, _}, WMLoop) ->
-    %% To get here we know Prefix matches the beginning of the path
-    RawPath = Req:get(raw_path),
-    Path = case Prefix of
-               "" -> RawPath;
-               _  -> string:substr(RawPath, string:len(Prefix) + 2)
-           end,
-    Redirect = fun(L) -> {301, [{"Location", Prefix ++ L}], ""} end,
+respond(Req, LocalPaths, WMLoop) ->
+    Path = Req:get(raw_path),
+    Redirect = fun(L) -> {301, [{"Location", L}], ""} end,
     case Path of
         "/api/" ++ Rest when length(Rest) > 0 ->
-            WMLoop(PL, Req);
+            WMLoop(Req);
         "" ->
             Req:respond(Redirect("/"));
         "/mgmt/" ->
@@ -127,28 +115,8 @@ setup_wm_logging() ->
             webmachine_sup:start_logger(LogDir)
     end.
 
-%% This doesn't *entirely* seem to work. It fails to load a non-existent
-%% image which seems to partly break it, but some stuff is usable.
-setup_wm_trace_app() ->
-    Loop = rabbit_webmachine:makeloop([{["wmtrace", '*'],
-                                       wmtrace_resource,
-                                       [{trace_dir, "/tmp"}]}]),
-    rabbit_mochiweb:register_static_context(
-      rabbit_mgmt_trace_static, "wmtrace/static", ?MODULE,
-      "deps/webmachine/webmachine/priv/trace", none),
-    rabbit_mochiweb:register_context_handler(rabbit_mgmt_trace,
-                                             "wmtrace", Loop,
-                                             "Webmachine tracer").
+log_startup(Listener) ->
+    rabbit_log:info("Management plugin started. Port: ~w~n", [port(Listener)]).
 
-log_startup() ->
-    rabbit_log:info(
-      "Management plugin started. Port: ~w, path: ~s~n",
-      [get_port(), rabbit_mochiweb:context_path(?CONTEXT, "/")]).
-
-get_port() ->
-    case rabbit_mochiweb:context_listener(?CONTEXT) of
-        undefined ->
-            exit(rabbit_mochiweb_listener_not_configured);
-        {_Instance, Options} ->
-            proplists:get_value(port, Options)
-    end.
+port(Listener) ->
+    proplists:get_value(port, Listener).
