@@ -11,36 +11,37 @@
 %%   The Original Code is RabbitMQ Management Plugin.
 %%
 %%   The Initial Developer of the Original Code is VMware, Inc.
-%%   Copyright (c) 2010-2012 VMware, Inc.  All rights reserved.
+%%   Copyright (c) 2010-2013 VMware, Inc.  All rights reserved.
 %%
 
 -module(rabbit_mgmt_db).
 
 -include_lib("rabbit_common/include/rabbit.hrl").
 
--behaviour(gen_server).
+-behaviour(gen_server2).
 
 -export([start_link/0]).
 
--export([augment_exchanges/2, augment_queues/2,
+-export([augment_exchanges/2, augment_queues/2, augment_nodes/1,
          get_channels/2, get_connections/1,
          get_all_channels/1, get_all_connections/0,
          get_overview/1, get_overview/0]).
 
 -export([init/1, handle_call/3, handle_cast/2, handle_info/2, terminate/2,
-         code_change/3]).
+         code_change/3, handle_pre_hibernate/1]).
 
--import(rabbit_misc, [pget/3]).
+-import(rabbit_misc, [pget/3, pset/3]).
 
 -record(state, {tables, interval}).
 -define(FINE_STATS_TYPES, [channel_queue_stats, channel_exchange_stats,
                            channel_queue_exchange_stats]).
--define(TABLES, [queue_stats, connection_stats, channel_stats, consumers] ++
+-define(TABLES, [queue_stats, connection_stats, channel_stats,
+                 consumers, node_stats] ++
             ?FINE_STATS_TYPES).
 
 -define(DELIVER_GET, [deliver, deliver_no_ack, get, get_no_ack]).
 -define(FINE_STATS, [publish, ack, deliver_get, confirm,
-                     return_unroutable, return_not_delivered, redeliver] ++
+                     return_unroutable, redeliver] ++
             ?DELIVER_GET).
 
 -define(
@@ -88,10 +89,23 @@
 %%----------------------------------------------------------------------------
 
 start_link() ->
-    gen_server:start_link({global, ?MODULE}, ?MODULE, [], []).
+    %% When failing over it is possible that the mirrored_supervisor
+    %% might hear of the death of the old DB, and start a new one,
+    %% before the global name server notices. Therefore rather than
+    %% telling gen_server:start_link/4 to register it for us, we
+    %% invoke global:re_register_name/2 ourselves, and just steal the
+    %% name if it existed before. We therefore rely on
+    %% mirrored_supervisor to maintain the uniqueness of this process.
+    case gen_server2:start_link(?MODULE, [], []) of
+        {ok, Pid} -> yes = global:re_register_name(?MODULE, Pid),
+                     rabbit:force_event_refresh(),
+                     {ok, Pid};
+        Else      -> Else
+    end.
 
 augment_exchanges(Xs, Mode) -> safe_call({augment_exchanges, Xs, Mode}, Xs).
 augment_queues(Qs, Mode)    -> safe_call({augment_queues, Qs, Mode}, Qs).
+augment_nodes(Nodes)        -> safe_call({augment_nodes, Nodes}, Nodes).
 
 get_channels(Cs, Mode)      -> safe_call({get_channels, Cs, Mode}, Cs).
 get_connections(Cs)         -> safe_call({get_connections, Cs}, Cs).
@@ -106,15 +120,13 @@ safe_call(Term) -> safe_call(Term, []).
 
 safe_call(Term, Item) ->
     try
-        gen_server:call({global, ?MODULE}, Term, infinity)
+        gen_server2:call({global, ?MODULE}, Term, infinity)
     catch exit:{noproc, _} -> Item
     end.
 
 %%----------------------------------------------------------------------------
 pget(Key, List) -> pget(Key, List, unknown).
 
-pset(Key, Value, List) -> [{Key, Value} | proplists:delete(Key, List)].
-
 id(Pid) when is_pid(Pid) -> Pid;
 id(List) -> pget(pid, List).
 
@@ -199,25 +211,32 @@ if_unknown(Val,    _Def) -> Val.
 %%----------------------------------------------------------------------------
 
 init([]) ->
-    rabbit:force_event_refresh(),
+    %% When Rabbit is overloaded, it's usually especially important
+    %% that the management plugin work.
+    process_flag(priority, high),
     {ok, Interval} = application:get_env(rabbit, collect_statistics_interval),
     rabbit_log:info("Statistics database started.~n"),
     {ok, #state{interval = Interval,
                 tables = orddict:from_list(
-                           [{Key, ets:new(anon, [private, ordered_set])} ||
-                               Key <- ?TABLES])}}.
+                           [{Key, ets:new(rabbit_mgmt_db,
+                                          [private, ordered_set])} ||
+                               Key <- ?TABLES])}, hibernate,
+     {backoff, ?HIBERNATE_AFTER_MIN, ?HIBERNATE_AFTER_MIN, ?DESIRED_HIBERNATE}}.
 
 handle_call({augment_exchanges, Xs, basic}, _From, State) ->
-    {reply, exchange_stats(Xs, ?FINE_STATS_EXCHANGE_LIST, State), State};
+    reply(exchange_stats(Xs, ?FINE_STATS_EXCHANGE_LIST, State), State);
 
 handle_call({augment_exchanges, Xs, full}, _From, State) ->
-    {reply, exchange_stats(Xs, ?FINE_STATS_EXCHANGE_DETAIL, State), State};
+    reply(exchange_stats(Xs, ?FINE_STATS_EXCHANGE_DETAIL, State), State);
 
 handle_call({augment_queues, Qs, basic}, _From, State) ->
-    {reply, list_queue_stats(Qs, State), State};
+    reply(list_queue_stats(Qs, State), State);
 
 handle_call({augment_queues, Qs, full}, _From, State) ->
-    {reply, detail_queue_stats(Qs, State), State};
+    reply(detail_queue_stats(Qs, State), State);
+
+handle_call({augment_nodes, Nodes}, _From, State) ->
+    {reply, node_stats(Nodes, State), State};
 
 handle_call({get_channels, Names, Mode}, _From,
             State = #state{tables = Tables}) ->
@@ -226,13 +245,13 @@ handle_call({get_channels, Names, Mode}, _From,
                  basic -> list_channel_stats(Chans, State);
                  full  -> detail_channel_stats(Chans, State)
              end,
-    {reply, lists:map(fun result_or_error/1, Result), State};
+    reply(lists:map(fun result_or_error/1, Result), State);
 
 handle_call({get_connections, Names}, _From,
             State = #state{tables = Tables}) ->
     Conns = created_event(Names, connection_stats, Tables),
     Result = connection_stats(Conns, State),
-    {reply, lists:map(fun result_or_error/1, Result), State};
+    reply(lists:map(fun result_or_error/1, Result), State);
 
 handle_call({get_all_channels, Mode}, _From, State = #state{tables = Tables}) ->
     Chans = created_events(channel_stats, Tables),
@@ -240,11 +259,11 @@ handle_call({get_all_channels, Mode}, _From, State = #state{tables = Tables}) ->
                  basic -> list_channel_stats(Chans, State);
                  full  -> detail_channel_stats(Chans, State)
              end,
-    {reply, Result, State};
+    reply(Result, State);
 
 handle_call(get_all_connections, _From, State = #state{tables = Tables}) ->
     Conns = created_events(connection_stats, Tables),
-    {reply, connection_stats(Conns, State), State};
+    reply(connection_stats(Conns, State), State);
 
 handle_call({get_overview, User}, _From, State = #state{tables = Tables}) ->
     VHosts = case User of
@@ -254,7 +273,8 @@ handle_call({get_overview, User}, _From, State = #state{tables = Tables}) ->
     Qs0 = [rabbit_mgmt_format:queue(Q) || V <- VHosts,
                                           Q <- rabbit_amqqueue:list(V)],
     Qs1 = basic_queue_stats(Qs0, State),
-    Totals = sum(Qs1, ?OVERVIEW_QUEUE_STATS),
+    QueueTotals = sum(Qs1, ?OVERVIEW_QUEUE_STATS),
+
     Filter = fun(Id, Name) ->
                      lists:member(pget(vhost, pget(Name, Id)), VHosts)
              end,
@@ -267,21 +287,40 @@ handle_call({get_overview, User}, _From, State = #state{tables = Tables}) ->
         end,
     Publish = F(channel_exchange_stats, exchange),
     Consume = F(channel_queue_stats, queue_details),
-    {reply, [{message_stats, Publish ++ Consume},
-             {queue_totals, Totals}], State};
+
+    F2 = case User of
+             all -> fun (L) -> length(L) end;
+             _   -> fun (L) -> length(rabbit_mgmt_util:filter_user(L, User)) end
+         end,
+    %% Filtering out the user's consumers would be rather expensive so let's
+    %% just not show it
+    Consumers = case User of
+                    all -> [{consumers,
+                             ets:info(orddict:fetch(consumers, Tables), size)}];
+                    _   -> []
+                end,
+    ObjectTotals = Consumers ++
+        [{queues,      length(Qs0)},
+         {exchanges,   length([X || V <- VHosts,
+                                    X <- rabbit_exchange:list(V)])},
+         {connections, F2(created_events(connection_stats, Tables))},
+         {channels,    F2(created_events(channel_stats, Tables))}],
+    reply([{message_stats, Publish ++ Consume},
+           {queue_totals,  QueueTotals},
+           {object_totals, ObjectTotals}], State);
 
 handle_call(_Request, _From, State) ->
-    {reply, not_understood, State}.
+    reply(not_understood, State).
 
 handle_cast({event, Event}, State) ->
     handle_event(Event, State),
-    {noreply, State};
+    noreply(State);
 
 handle_cast(_Request, State) ->
-    {noreply, State}.
+    noreply(State).
 
 handle_info(_Info, State) ->
-    {noreply, State}.
+    noreply(State).
 
 terminate(_Arg, _State) ->
     ok.
@@ -289,26 +328,27 @@ terminate(_Arg, _State) ->
 code_change(_OldVsn, State, _Extra) ->
     {ok, State}.
 
-%%----------------------------------------------------------------------------
+reply(Reply, NewState) -> {reply, Reply, NewState, hibernate}.
+noreply(NewState) -> {noreply, NewState, hibernate}.
 
-handle_event(#event{type = queue_created, props = Props}, State) ->
-    QName = pget(name, Props),
-    case pget(synchronised_slave_pids, Props) of
-        SSPids when is_list(SSPids) ->
-            [handle_slave_synchronised(QName, SSPid, State) || SSPid <- SSPids];
-        _ ->
-            ok
-    end,
-    {ok, State};
+handle_pre_hibernate(State) ->
+    %% rabbit_event can end up holding on to some memory after a busy
+    %% workout, but it's not a gen_server so we can't make it
+    %% hibernate. The best we can do is forcibly GC it here (if
+    %% rabbit_mgmt_db is hibernating the odds are rabbit_event is
+    %% quiescing in some way too).
+    rpc:multicall(
+      rabbit_mnesia:cluster_nodes(running), rabbit_mgmt_db_handler, gc, []),
+    {hibernate, State}.
+
+%%----------------------------------------------------------------------------
 
 handle_event(#event{type = queue_stats, props = Stats, timestamp = Timestamp},
              State) ->
     handle_stats(queue_stats, Stats, Timestamp,
                  [{fun rabbit_mgmt_format:properties/1,[backing_queue_status]},
                   {fun rabbit_mgmt_format:timestamp/1, [idle_since]}],
-                 [messages, messages_ready, messages_unacknowledged], State),
-    prune_synchronised_slaves(
-      true, pget(name, Stats), pget(slave_pids, Stats), State);
+                 [messages, messages_ready, messages_unacknowledged], State);
 
 handle_event(Event = #event{type = queue_deleted}, State) ->
     handle_deleted(queue_stats, Event, State);
@@ -316,7 +356,7 @@ handle_event(Event = #event{type = queue_deleted}, State) ->
 handle_event(#event{type = connection_created, props = Stats}, State) ->
     handle_created(
       connection_stats, Stats,
-      [{fun rabbit_mgmt_format:addr/1,         [address, peer_address]},
+      [{fun rabbit_mgmt_format:addr/1,         [host, peer_host]},
        {fun rabbit_mgmt_format:port/1,         [port, peer_port]},
        {fun rabbit_mgmt_format:protocol/1,     [protocol]},
        {fun rabbit_mgmt_format:amqp_table/1,   [client_properties]}], State);
@@ -357,22 +397,23 @@ handle_event(#event{type = consumer_deleted, props = Props}, State) ->
     handle_consumer(fun(Table, Id, _P) -> ets:delete(Table, Id) end,
                     Props, State);
 
-handle_event(#event{type = queue_slave_synchronised, props = Props}, State) ->
-    handle_slave_synchronised(pget(name, Props), pget(pid, Props), State);
-
-handle_event(#event{type = queue_slave_promoted, props = Props}, State) ->
-    handle_slave_promoted(pget(name, Props), pget(pid, Props), State);
-
 handle_event(#event{type = queue_mirror_deaths, props = Props},
-             State = #state{tables = Tables}) ->
+             #state{tables = Tables}) ->
     Dead = pget(pids, Props),
     Table = orddict:fetch(queue_stats, Tables),
-    %% This is of course slow. It would be faster if the queue stats
-    %% were keyed off queue name as well. But that's a big change, and
-    %% this doesn't happen very often.
-    prune_slaves(ets:match(Table, {{'$1', stats}, '$2', '$3'}, 100),
-                 Dead, Table),
-    prune_synchronised_slaves(false, pget(name, Props), Dead, State);
+    %% Only the master can be in the DB, but it's easier just to
+    %% delete all of them
+    [ets:delete(Table, {Pid, stats}) || Pid <- Dead];
+
+%% TODO: we don't clear up after dead nodes here - this is a very tiny
+%% leak every time a node is permanently removed from the cluster. Do
+%% we care?
+handle_event(#event{type = node_stats, props = Stats, timestamp = Timestamp},
+             State = #state{tables = Tables}) ->
+    Table = orddict:fetch(node_stats, Tables),
+    ets:insert(Table, {{pget(name, Stats), stats},
+                       proplists:delete(name, Stats), Timestamp}),
+    {ok, State};
 
 handle_event(_Event, State) ->
     {ok, State}.
@@ -403,10 +444,8 @@ handle_stats(TName, Stats0, Timestamp, Funs,
 handle_deleted(TName, #event{props = Props}, State = #state{tables = Tables}) ->
     Table = orddict:fetch(TName, Tables),
     Pid = pget(pid, Props),
-    Name = pget(name, Props),
     ets:delete(Table, {id(Pid), create}),
     ets:delete(Table, {id(Pid), stats}),
-    ets:delete(Table, {Name, synchronised_slaves}),
     {ok, State}.
 
 handle_consumer(Fun, Props,
@@ -416,51 +455,6 @@ handle_consumer(Fun, Props,
     Fun(Table, {pget(queue, P), pget(channel, P)}, P),
     {ok, State}.
 
-handle_slave_synchronised(QName, SSPid, State) ->
-    SSNode = node(SSPid),
-    update_synchronised_slaves(
-      fun (SSNodes) -> case lists:member(SSNode, SSNodes) of
-                           true  -> SSNodes;
-                           false -> [SSNode | SSNodes]
-                       end
-      end, QName, State).
-
-handle_slave_promoted(QName, NewMPid, State) ->
-    update_synchronised_slaves(
-      fun (SSNodes) -> SSNodes -- [node(NewMPid)] end, QName, State).
-
-prune_synchronised_slaves(Member, QName, SPids, State) ->
-    SNodes = [node(SPid) || SPid <- SPids],
-    update_synchronised_slaves(
-      fun (SSNodes) ->
-              lists:filter(fun (S) -> lists:member(S, SNodes) =:= Member end,
-                           SSNodes)
-      end, QName, State).
-
-update_synchronised_slaves(Fun, QName, State = #state{tables = Tables}) ->
-    Table = orddict:fetch(queue_stats, Tables),
-    New = case ets:lookup(Table, {QName, synchronised_slaves}) of
-              []             -> Fun([]);
-              [{_, SSNodes}] -> Fun(SSNodes)
-          end,
-    ets:insert(Table, {{QName, synchronised_slaves}, New}),
-    {ok, State}.
-
-prune_slaves('$end_of_table', _Dead, _Table) -> ok;
-
-prune_slaves({Matches, Continuation}, Dead, Table) ->
-    [prune_slaves0(M, Dead, Table) || M <- Matches],
-    prune_slaves(ets:match(Continuation), Dead, Table).
-
-prune_slaves0([Pid, Stats, Timestamp], Dead, Table) ->
-    Old = pget(slave_pids, Stats),
-    New = Old -- Dead,
-    case New of
-        Old -> ok;
-        _   -> NewStats = pset(slave_pids, New, Stats),
-               ets:insert(Table, {{Pid, stats}, NewStats, Timestamp})
-    end.
-
 handle_fine_stats(Type, Props, Timestamp, State = #state{tables = Tables}) ->
     case pget(Type, Props) of
         unknown ->
@@ -537,11 +531,11 @@ merge_stats(Objs, Funs) ->
     [lists:foldl(fun (Fun, Props) -> Fun(Props) ++ Props end, Obj, Funs)
      || Obj <- Objs].
 
-basic_stats_fun(Type, State = #state{tables = Tables}) ->
+basic_stats_fun(Type, IdProperty, State = #state{tables = Tables}) ->
     Table = orddict:fetch(Type, Tables),
     fun (Props) ->
-            zero_old_rates(lookup_element(Table, {pget(pid, Props), stats}),
-                           State)
+            zero_old_rates(
+              lookup_element(Table, {pget(IdProperty, Props), stats}), State)
     end.
 
 fine_stats_fun(FineSpecs, State) ->
@@ -586,18 +580,6 @@ consumer_details_fun(PatternFun, State = #state{tables = Tables}) ->
                                   ets:match(Table, {Pattern, '$1'}))]}]
     end.
 
-synchronised_slaves_fun(#state{tables = Tables}) ->
-    Table = orddict:fetch(queue_stats, Tables),
-
-    fun (Props) -> QName = rabbit_misc:r(pget(vhost, Props), queue,
-                                         pget(name, Props)),
-                   Key = {QName, synchronised_slaves},
-                   case ets:lookup(Table, Key) of
-                       []       -> [];
-                       [{_, N}] -> [{synchronised_slave_nodes, N}]
-                   end
-    end.
-
 zero_old_rates(Stats, State) -> [maybe_zero_rate(S, State) || S <- Stats].
 
 maybe_zero_rate({Key, Val}, #state{interval = Interval}) ->
@@ -647,15 +629,15 @@ augment_channel_pid(Pid, #state{tables = Tables}) ->
     [{name,            pget(name,   Ch)},
      {number,          pget(number, Ch)},
      {connection_name, pget(name,         Conn)},
-     {peer_address,    pget(peer_address, Conn)},
-     {peer_port,       pget(peer_port,    Conn)}].
+     {peer_port,       pget(peer_port,    Conn)},
+     {peer_host,       pget(peer_host,    Conn)}].
 
 augment_connection_pid(Pid, #state{tables = Tables}) ->
     Conn = lookup_element(orddict:fetch(connection_stats, Tables),
                           {Pid, create}),
     [{name,         pget(name,         Conn)},
-     {peer_address, pget(peer_address, Conn)},
-     {peer_port,    pget(peer_port,    Conn)}].
+     {peer_port,    pget(peer_port,    Conn)},
+     {peer_host,    pget(peer_host,    Conn)}].
 
 augment_queue_pid(Pid, _State) ->
     %% TODO This should be in rabbit_amqqueue?
@@ -685,29 +667,31 @@ detail_queue_stats(Objs, State) ->
                       queue_funs(State))).
 
 queue_funs(State) ->
-    [basic_stats_fun(queue_stats, State), augment_msg_stats_fun(State),
-     synchronised_slaves_fun(State)].
+    [basic_stats_fun(queue_stats, pid, State), augment_msg_stats_fun(State)].
 
 exchange_stats(Objs, FineSpecs, State) ->
     merge_stats(Objs, [fine_stats_fun(FineSpecs, State),
                        augment_msg_stats_fun(State)]).
 
 connection_stats(Objs, State) ->
-    merge_stats(Objs, [basic_stats_fun(connection_stats, State),
+    merge_stats(Objs, [basic_stats_fun(connection_stats, pid, State),
                        augment_msg_stats_fun(State)]).
 
 list_channel_stats(Objs, State) ->
-    merge_stats(Objs, [basic_stats_fun(channel_stats, State),
+    merge_stats(Objs, [basic_stats_fun(channel_stats, pid, State),
                        fine_stats_fun(?FINE_STATS_CHANNEL_LIST, State),
                        augment_msg_stats_fun(State)]).
 
 detail_channel_stats(Objs, State) ->
-    merge_stats(Objs, [basic_stats_fun(channel_stats, State),
+    merge_stats(Objs, [basic_stats_fun(channel_stats, pid, State),
                        consumer_details_fun(
                          fun (Props) -> {'_', pget(pid, Props)} end, State),
                        fine_stats_fun(?FINE_STATS_CHANNEL_DETAIL, State),
                        augment_msg_stats_fun(State)]).
 
+node_stats(Objs, State) ->
+    merge_stats(Objs, [basic_stats_fun(node_stats, name, State)]).
+
 %%----------------------------------------------------------------------------
 
 %% We do this when retrieving the queue record rather than when
