@@ -11,7 +11,7 @@
 %%   The Original Code is RabbitMQ Management Plugin.
 %%
 %%   The Initial Developer of the Original Code is VMware, Inc.
-%%   Copyright (c) 2010-2012 VMware, Inc.  All rights reserved.
+%%   Copyright (c) 2010-2013 VMware, Inc.  All rights reserved.
 %%
 
 -module(rabbit_mgmt_dispatcher).
@@ -42,6 +42,13 @@ dispatcher() ->
      {["extensions"],                                              rabbit_mgmt_wm_extensions, []},
      {["all-configuration"],                                       rabbit_mgmt_wm_definitions, []}, %% This was the old name, let's not break things gratuitously.
      {["definitions"],                                             rabbit_mgmt_wm_definitions, []},
+     {["parameters"],                                              rabbit_mgmt_wm_parameters, []},
+     {["parameters", component],                                   rabbit_mgmt_wm_parameters, []},
+     {["parameters", component, vhost],                            rabbit_mgmt_wm_parameters, []},
+     {["parameters", component, vhost, name],                      rabbit_mgmt_wm_parameter, []},
+     {["policies"],                                                rabbit_mgmt_wm_policies, []},
+     {["policies", vhost],                                         rabbit_mgmt_wm_policies, []},
+     {["policies", vhost, name],                                   rabbit_mgmt_wm_policy, []},
      {["connections"],                                             rabbit_mgmt_wm_connections, []},
      {["connections", connection],                                 rabbit_mgmt_wm_connection, []},
      {["connections", connection, "channels"],                     rabbit_mgmt_wm_connection_channels, []},
