@@ -11,20 +11,22 @@
 %%   The Original Code is RabbitMQ Management Plugin.
 %%
 %%   The Initial Developer of the Original Code is VMware, Inc.
-%%   Copyright (c) 2010-2012 VMware, Inc.  All rights reserved.
+%%   Copyright (c) 2010-2013 VMware, Inc.  All rights reserved.
 %%
 
 -module(rabbit_mgmt_format).
 
 -export([format/2, print/2, remove/1, ip/1, ipb/1, amqp_table/1, tuple/1]).
--export([timestamp/1, timestamp_ms/1, strip_pids/1]).
+-export([parameter/1, timestamp/1, timestamp_ms/1, strip_pids/1]).
 -export([node_from_pid/1, protocol/1, resource/1, queue/1]).
 -export([exchange/1, user/1, internal_user/1, binding/1, url/2]).
--export([pack_binding_props/2, unpack_binding_props/1, tokenise/1]).
+-export([pack_binding_props/2, tokenise/1]).
 -export([to_amqp_table/1, listener/1, properties/1, basic_properties/1]).
 -export([record/2, to_basic_properties/1]).
 -export([addr/1, port/1]).
 
+-import(rabbit_misc, [pget/2, pset/3]).
+
 -include_lib("rabbit_common/include/rabbit.hrl").
 -include_lib("rabbit_common/include/rabbit_framing.hrl").
 
@@ -73,8 +75,8 @@ ip(IP)      -> list_to_binary(rabbit_misc:ntoa(IP)).
 ipb(unknown) -> unknown;
 ipb(IP)      -> list_to_binary(rabbit_misc:ntoab(IP)).
 
-addr(Addr) when is_list(Addr); is_atom(Addr) -> print("~s", Addr);
-addr(Addr) when is_tuple(Addr)               -> ip(Addr).
+addr(S)    when is_list(S); is_atom(S); is_binary(S) -> print("~s", S);
+addr(Addr) when is_tuple(Addr)                       -> ip(Addr).
 
 port(Port) when is_number(Port) -> Port;
 port(Port)                      -> print("~w", Port).
@@ -102,6 +104,8 @@ utf8_safe(V) ->
             <<"Invalid UTF-8, base64 is: ", Enc/binary>>
     end.
 
+parameter(P) -> pset(value, rabbit_misc:term_to_json(pget(value, P)), P).
+
 tuple(unknown)                    -> unknown;
 tuple(Tuple) when is_tuple(Tuple) -> [tuple(E) || E <- tuple_to_list(Tuple)];
 tuple(Term)                       -> Term.
@@ -139,6 +143,9 @@ resource(_, unknown) ->
 resource(NameAs, #resource{name = Name, virtual_host = VHost}) ->
     [{NameAs, Name}, {vhost, VHost}].
 
+policy('')     -> [];
+policy(Policy) -> [{policy, Policy}].
+
 internal_user(User) ->
     [{name,          User#internal_user.username},
      {password_hash, base64:encode(User#internal_user.password_hash)},
@@ -160,55 +167,22 @@ listener(#listener{node = Node, protocol = Protocol,
      {port, Port}].
 
 pack_binding_props(<<"">>, []) ->
-    <<"_">>;
+    <<"~">>;
+pack_binding_props(Key, []) ->
+    list_to_binary(quote_binding(Key));
 pack_binding_props(Key, Args) ->
-    Dict = dict:from_list([{K, V} || {K, _, V} <- Args]),
-    ArgsKeys = lists:sort(dict:fetch_keys(Dict)),
-    PackedArgs =
-        string:join(
-          [quote_binding(K) ++ "_" ++
-               quote_binding(dict:fetch(K, Dict)) || K <- ArgsKeys],
-          "_"),
-    list_to_binary(quote_binding(Key) ++
-                       case PackedArgs of
-                           "" -> "";
-                           _  -> "_" ++ PackedArgs
-                       end).
+    ArgsEnc = rabbit_mgmt_wm_binding:args_hash(Args),
+    list_to_binary(quote_binding(Key) ++ "~" ++ quote_binding(ArgsEnc)).
 
 quote_binding(Name) ->
-    binary_to_list(
-      iolist_to_binary(
-        re:replace(mochiweb_util:quote_plus(Name), "_", "%5F", [global]))).
+    re:replace(mochiweb_util:quote_plus(Name), "~", "%7E", [global]).
 
-unpack_binding_props(B) when is_binary(B) ->
-    unpack_binding_props(binary_to_list(B));
-unpack_binding_props(Str) ->
-    unpack_binding_props0(tokenise(Str)).
-
-unpack_binding_props0([Key | Args]) ->
-    try
-        {unquote_binding(Key), to_amqp_table(unpack_binding_args(Args))}
-    catch E -> E
-    end;
-unpack_binding_props0([]) ->
-    {bad_request, empty_properties}.
-
-unpack_binding_args([]) ->
-    [];
-unpack_binding_args([K]) ->
-    throw({bad_request, {no_value, K}});
-unpack_binding_args([K, V | Rest]) ->
-    [{unquote_binding(K), unquote_binding(V)} | unpack_binding_args(Rest)].
-
-unquote_binding(Name) ->
-    list_to_binary(mochiweb_util:unquote(Name)).
-
-%% Unfortunately string:tokens("foo__bar", "_"). -> ["foo","bar"], we lose
-%% the fact that there's a double _.
+%% Unfortunately string:tokens("foo~~bar", "~"). -> ["foo","bar"], we lose
+%% the fact that there's a double ~.
 tokenise("") ->
     [];
 tokenise(Str) ->
-    Count = string:cspan(Str, "_"),
+    Count = string:cspan(Str, "~"),
     case length(Str) of
         Count -> [Str];
         _     -> [string:sub_string(Str, 1, Count) |
@@ -227,19 +201,23 @@ to_amqp_table_row(K, V) ->
 to_amqp_array(L) ->
     [type_val(I) || I <- L].
 
-type_val({struct, M})         -> {table,   to_amqp_table(M)};
-type_val(L) when is_list(L)   -> {array,   to_amqp_array(L)};
-type_val(X) when is_binary(X) -> {longstr, X};
-type_val(X) when is_number(X) -> {long,    X};
-type_val(null)                -> throw({error, null_not_allowed});
-type_val(X)                   -> throw({error, {unhandled_type, X}}).
+type_val({struct, M})          -> {table,   to_amqp_table(M)};
+type_val(L) when is_list(L)    -> {array,   to_amqp_array(L)};
+type_val(X) when is_binary(X)  -> {longstr, X};
+type_val(X) when is_integer(X) -> {long,    X};
+type_val(X) when is_number(X)  -> {double,  X};
+type_val(true)                 -> {bool, true};
+type_val(false)                -> {bool, false};
+type_val(null)                 -> throw({error, null_not_allowed});
+type_val(X)                    -> throw({error, {unhandled_type, X}}).
 
 url(Fmt, Vals) ->
     print(Fmt, [mochiweb_util:quote_plus(V) || V <- Vals]).
 
 exchange(X) ->
     format(X, [{fun resource/1,   [name]},
-               {fun amqp_table/1, [arguments]}]).
+               {fun amqp_table/1, [arguments]},
+               {fun policy/1,     [policy]}]).
 
 %% We get queues using rabbit_amqqueue:list/1 rather than :info_all/1 since
 %% the latter wakes up each queue. Therefore we have a record rather than a
@@ -249,7 +227,7 @@ queue(#amqqueue{name            = Name,
                 auto_delete     = AutoDelete,
                 exclusive_owner = ExclusiveOwner,
                 arguments       = Arguments,
-                pid             = Pid }) ->
+                pid             = Pid}) ->
     format(
       [{name,        Name},
        {durable,     Durable},
@@ -257,8 +235,9 @@ queue(#amqqueue{name            = Name,
        {owner_pid,   ExclusiveOwner},
        {arguments,   Arguments},
        {pid,         Pid}],
-      [{fun resource/1,     [name]},
-       {fun amqp_table/1,   [arguments]}]).
+      [{fun resource/1,   [name]},
+       {fun amqp_table/1, [arguments]},
+       {fun policy/1,     [policy]}]).
 
 %% We get bindings using rabbit_binding:list_*/1 rather than :info_all/1 since
 %% there are no per-exchange / queue / etc variants for the latter. Therefore
@@ -319,7 +298,9 @@ strip_pids(Item = [T | _]) when is_tuple(T) ->
     format(Item,
            [{fun node_from_pid/1, [pid]},
             {fun remove/1,        ?PIDS_TO_STRIP},
-            {nodes_from_pids(slave_nodes), [slave_pids]}
+            {nodes_from_pids(slave_nodes), [slave_pids]},
+            {nodes_from_pids(synchronised_slave_nodes),
+             [synchronised_slave_pids]}
            ]);
 
 strip_pids(Items) -> [strip_pids(I) || I <- Items].
