@@ -11,7 +11,7 @@
 %%   The Original Code is RabbitMQ Management Plugin.
 %%
 %%   The Initial Developer of the Original Code is VMware, Inc.
-%%   Copyright (c) 2010-2012 VMware, Inc.  All rights reserved.
+%%   Copyright (c) 2010-2013 VMware, Inc.  All rights reserved.
 %%
 
 -module(rabbit_mgmt_util).
@@ -19,7 +19,7 @@
 %% TODO sort all this out; maybe there's scope for rabbit_mgmt_request?
 
 -export([is_authorized/2, is_authorized_admin/2, vhost/1]).
--export([is_authorized_vhost/2, is_authorized/3, is_authorized_user/3,
+-export([is_authorized_vhost/2, is_authorized_user/3,
          is_authorized_monitor/2]).
 -export([bad_request/3, bad_request_exception/4, id/2, parse_bool/1,
          parse_int/1]).
@@ -27,10 +27,10 @@
 -export([with_channel/4, with_channel/5]).
 -export([props_to_method/2, props_to_method/4]).
 -export([all_or_one_vhost/2, http_to_amqp/5, reply/3, filter_vhost/3]).
--export([filter_conn_ch_list/3, with_decode/5, decode/1, decode/2, redirect/2,
-         args/1]).
+-export([filter_conn_ch_list/3, filter_user/2]).
+-export([with_decode/5, decode/1, decode/2, redirect/2, args/1]).
 -export([reply_list/3, reply_list/4, sort_list/2, destination_type/1]).
--export([post_respond/1, columns/1, want_column/2, is_monitor/1]).
+-export([post_respond/1, columns/1, is_monitor/1]).
 -export([list_visible_vhosts/1, b64decode_or_throw/1]).
 
 -import(rabbit_misc, [pget/2, pget/3]).
@@ -43,53 +43,67 @@
 %%--------------------------------------------------------------------
 
 is_authorized(ReqData, Context) ->
-    is_authorized(ReqData, Context, fun(_) -> true end).
+    is_authorized(ReqData, Context, '', fun(_) -> true end).
 
 is_authorized_admin(ReqData, Context) ->
     is_authorized(ReqData, Context,
+                  <<"Not administrator user">>,
                   fun(#user{tags = Tags}) -> is_admin(Tags) end).
 
 is_authorized_monitor(ReqData, Context) ->
     is_authorized(ReqData, Context,
+                  <<"Not monitor user">>,
                   fun(#user{tags = Tags}) -> is_monitor(Tags) end).
 
 is_authorized_vhost(ReqData, Context) ->
-    is_authorized(
-      ReqData, Context,
-      fun(User) ->
-              case vhost(ReqData) of
-                  not_found -> true;
-                  none      -> true;
-                  V         -> lists:member(V, list_login_vhosts(User))
-              end
-      end).
+    is_authorized(ReqData, Context,
+                  <<"User not authorised to access virtual host">>,
+                  fun(User) ->
+                          case vhost(ReqData) of
+                              not_found -> true;
+                              none      -> true;
+                              V         -> lists:member(
+                                             V, list_login_vhosts(User))
+                          end
+                  end).
 
 %% Used for connections / channels. A normal user can only see / delete
 %% their own stuff. Monitors can see other users' and delete their
 %% own. Admins can do it all.
 is_authorized_user(ReqData, Context, Item) ->
-    is_authorized(
-      ReqData, Context,
-      fun(#user{username = Username, tags = Tags}) ->
-              case wrq:method(ReqData) of
-                  'DELETE' -> is_admin(Tags);
-                  _        -> is_monitor(Tags)
-              end orelse Username == pget(user, Item)
-      end).
+    is_authorized(ReqData, Context,
+                  <<"User not authorised to access object">>,
+                  fun(#user{username = Username, tags = Tags}) ->
+                          case wrq:method(ReqData) of
+                              'DELETE' -> is_admin(Tags);
+                              _        -> is_monitor(Tags)
+                          end orelse Username == pget(user, Item)
+                  end).
 
-is_authorized(ReqData, Context, Fun) ->
-    %% Note that we've already done authentication in the Mochiweb
-    %% world, so we know we're authorised. Unfortunately we can't
-    %% influence ReqData or Context from Mochiweb, so we have to
-    %% invoke check_user_pass_login again to see what kind of user we
-    %% have.
-    [Username, Password] = rabbit_mochiweb_util:parse_auth_header(
-                             wrq:get_req_header("authorization", ReqData)),
-    {ok, User = #user{tags = Tags}} =
-        rabbit_access_control:check_user_pass_login(Username, Password),
-    case is_mgmt_user(Tags) andalso Fun(User) of
-        true  -> {true, ReqData, Context#context{user = User}};
-        false -> {?AUTH_REALM, ReqData, Context}
+is_authorized(ReqData, Context, ErrorMsg, Fun) ->
+    ErrFun = fun (Msg) -> not_authorised(Msg, ReqData, Context) end,
+    case rabbit_mochiweb_util:parse_auth_header(
+           wrq:get_req_header("authorization", ReqData)) of
+        [Username, Password] ->
+            case rabbit_access_control:check_user_pass_login(
+                   Username, Password) of
+                {ok, User = #user{tags = Tags}} ->
+                    case is_mgmt_user(Tags) of
+                        true  -> case Fun(User) of
+                                     true  -> {true, ReqData,
+                                               Context#context{
+                                                 user     = User,
+                                                 password = Password}};
+                                     false -> ErrFun(ErrorMsg)
+                                 end;
+                        false -> ErrFun(<<"Not management user">>)
+                    end;
+                _ ->
+                    ErrFun(<<"Login failed">>)
+
+            end;
+        _ ->
+            {?AUTH_REALM, ReqData, Context}
     end.
 
 vhost(ReqData) ->
@@ -111,8 +125,7 @@ reply(Facts, ReqData, Context) ->
     ReqData1 = wrq:set_resp_header("Cache-Control", "no-cache", ReqData),
     try
         {mochijson2:encode(Facts), ReqData1, Context}
-    catch
-        exit:{json_encode, E} ->
+    catch exit:{json_encode, E} ->
             Error = iolist_to_binary(
                       io_lib:format("JSON encode error: ~p", [E])),
             Reason = iolist_to_binary(
@@ -131,8 +144,7 @@ reply_list(Facts, DefaultSorts, ReqData, Context) ->
             wrq:get_qs_value("sort_reverse", ReqData)),
           ReqData, Context).
 
-sort_list(Facts, Sorts) ->
-    sort_list(Facts, Sorts, undefined, false).
+sort_list(Facts, Sorts) -> sort_list(Facts, Sorts, undefined, false).
 
 sort_list(Facts, DefaultSorts, Sort, Reverse) ->
     SortList = case Sort of
@@ -159,30 +171,49 @@ get_dotted_value(Key, Item) ->
 get_dotted_value0([Key], Item) ->
     %% Put "nothing" before everything else, in number terms it usually
     %% means 0.
-    pget(list_to_atom(Key), Item, 0);
+    pget_bin(list_to_binary(Key), Item, 0);
 get_dotted_value0([Key | Keys], Item) ->
-    get_dotted_value0(Keys, pget(list_to_atom(Key), Item, [])).
+    get_dotted_value0(Keys, pget_bin(list_to_binary(Key), Item, [])).
+
+pget_bin(Key, List, Default) ->
+    case lists:partition(fun ({K, _V}) -> a2b(K) =:= Key end, List) of
+        {[{_K, V}], _} -> V;
+        {[],        _} -> Default
+    end.
 
 extract_columns(Items, ReqData) ->
     Cols = columns(ReqData),
     [extract_column_items(Item, Cols) || Item <- Items].
 
+columns(ReqData) ->
+    case wrq:get_qs_value("columns", ReqData) of
+        undefined -> all;
+        Str       -> [[list_to_binary(T) || T <- string:tokens(C, ".")]
+                      || C <- string:tokens(Str, ",")]
+    end.
+
 extract_column_items(Item, all) ->
     Item;
 extract_column_items({struct, L}, Cols) ->
     extract_column_items(L, Cols);
 extract_column_items(Item = [T | _], Cols) when is_tuple(T) ->
-    [{K, extract_column_items(V, descend_columns(K, Cols))} ||
-        {K, V} <- Item, want_column(K, Cols)];
+    [{K, extract_column_items(V, descend_columns(a2b(K), Cols))} ||
+        {K, V} <- Item, want_column(a2b(K), Cols)];
 extract_column_items(L, Cols) when is_list(L) ->
     [extract_column_items(I, Cols) || I <- L];
 extract_column_items(O, _Cols) ->
     O.
 
-descend_columns(_K, [])                -> [];
-descend_columns(K, [[K] | _Rest])      -> all;
-descend_columns(K, [[K | K2] | Rest])  -> [K2 | descend_columns(K, Rest)];
-descend_columns(K, [[_K2 | _] | Rest]) -> descend_columns(K, Rest).
+want_column(_Col, all) -> true;
+want_column(Col, Cols) -> lists:any(fun([C|_]) -> C == Col end, Cols).
+
+descend_columns(_K, [])                   -> [];
+descend_columns( K, [[K]        | _Rest]) -> all;
+descend_columns( K, [[K   | K2] |  Rest]) -> [K2 | descend_columns(K, Rest)];
+descend_columns( K, [[_K2 | _ ] |  Rest]) -> descend_columns(K, Rest).
+
+a2b(A) when is_atom(A) -> list_to_binary(atom_to_list(A));
+a2b(B)                 -> B.
 
 bad_request(Reason, ReqData, Context) ->
     halt_response(400, bad_request, Reason, ReqData, Context).
@@ -307,38 +338,35 @@ parse_bool(true)        -> true;
 parse_bool(false)       -> false;
 parse_bool(V)           -> throw({error, {not_boolean, V}}).
 
-parse_int(I) when is_integer(I) ->
-    I;
-parse_int(F) when is_number(F) ->
-    trunc(F);
-parse_int(S) ->
-    try
-        list_to_integer(binary_to_list(S))
-    catch error:badarg ->
-        throw({error, {not_integer, S}})
-    end.
+parse_int(I) when is_integer(I) -> I;
+parse_int(F) when is_number(F)  -> trunc(F);
+parse_int(S)                    -> try
+                                       list_to_integer(binary_to_list(S))
+                                   catch error:badarg ->
+                                           throw({error, {not_integer, S}})
+                                   end.
 
 amqp_request(VHost, ReqData, Context, Method) ->
     amqp_request(VHost, ReqData, Context, node(), Method).
 
 amqp_request(VHost, ReqData, Context, Node, Method) ->
     with_channel(VHost, ReqData, Context, Node,
-                      fun (Ch) ->
-                              amqp_channel:call(Ch, Method),
-                              {true, ReqData, Context}
-                      end).
+                 fun (Ch) ->
+                         amqp_channel:call(Ch, Method),
+                         {true, ReqData, Context}
+                 end).
 
 with_channel(VHost, ReqData, Context, Fun) ->
     with_channel(VHost, ReqData, Context, node(), Fun).
 
 with_channel(VHost, ReqData,
-             Context = #context{user = #user {username = Username}},
+             Context = #context{user     = #user {username = Username},
+                                password = Password},
              Node, Fun) ->
     Params = #amqp_params_direct{username     = Username,
+                                 password     = Password,
                                  node         = Node,
                                  virtual_host = VHost},
-    %% We don't need to check the password here, we already did in
-    %% is_authorized/3.
     case amqp_connection:start(Params) of
         {ok, Conn} ->
             {ok, Ch} = amqp_connection:open_channel(Conn),
@@ -388,8 +416,10 @@ filter_vhost(List, _ReqData, Context) ->
     VHosts = list_login_vhosts(Context#context.user),
     [I || I <- List, lists:member(pget(vhost, I), VHosts)].
 
-filter_user(List, _ReqData,
-            #context{user = #user{username = Username, tags = Tags}}) ->
+filter_user(List, _ReqData, #context{user = User}) ->
+    filter_user(List, User).
+
+filter_user(List, #user{username = Username, tags = Tags}) ->
     case is_monitor(Tags) of
         true  -> List;
         false -> [I || I <- List, pget(user, I) == Username]
@@ -407,10 +437,9 @@ redirect(Location, ReqData) ->
     wrq:do_redirect(true,
                     wrq:set_resp_header("Location",
                                         binary_to_list(Location), ReqData)).
-args({struct, L}) ->
-    args(L);
-args(L) ->
-    rabbit_mgmt_format:to_amqp_table(L).
+
+args({struct, L}) -> args(L);
+args(L)           -> rabbit_mgmt_format:to_amqp_table(L).
 
 %% Make replying to a post look like anything else...
 post_respond({{halt, Code}, ReqData, Context}) ->
@@ -420,16 +449,6 @@ post_respond({JSON, ReqData, Context}) ->
              "content-type", "application/json",
              wrq:append_to_response_body(JSON, ReqData)), Context}.
 
-columns(ReqData) ->
-    case wrq:get_qs_value("columns", ReqData) of
-        undefined -> all;
-        Str       -> [[list_to_atom(T) || T <- string:tokens(C, ".")]
-                      || C <- string:tokens(Str, ",")]
-    end.
-
-want_column(_Col, all) -> true;
-want_column(Col, Cols) -> lists:any(fun([C|_]) -> C == Col end, Cols).
-
 is_admin(T)     -> intersects(T, [administrator]).
 is_monitor(T)   -> intersects(T, [administrator, monitoring]).
 is_mgmt_user(T) -> intersects(T, [administrator, monitoring, management]).
@@ -460,7 +479,8 @@ list_login_vhosts(User) ->
 %% Wow, base64:decode throws lots of weird errors. Catch and convert to one
 %% that will cause a bad_request.
 b64decode_or_throw(B64) ->
-    try base64:decode(B64)
+    try
+        base64:decode(B64)
     catch error:_ ->
             throw({error, {not_base64, B64}})
     end.
