@@ -11,7 +11,7 @@
 %%   The Original Code is RabbitMQ Management Plugin.
 %%
 %%   The Initial Developer of the Original Code is VMware, Inc.
-%%   Copyright (c) 2010-2012 VMware, Inc.  All rights reserved.
+%%   Copyright (c) 2010-2013 VMware, Inc.  All rights reserved.
 %%
 
 -module(rabbit_mgmt_wm_definitions).
@@ -59,7 +59,9 @@ to_json(ReqData, Context) ->
     rabbit_mgmt_util:reply(
       [{rabbit_version, list_to_binary(Vsn)}] ++
       filter(
-        [{users,       rabbit_mgmt_wm_users:users()},
+        [{parameters,  rabbit_mgmt_wm_parameters:basic(ReqData)},
+         {policies,    rabbit_mgmt_wm_policies:basic(ReqData)},
+         {users,       rabbit_mgmt_wm_users:users()},
          {vhosts,      rabbit_mgmt_wm_vhosts:basic()},
          {permissions, rabbit_mgmt_wm_permissions:permissions()},
          {queues,      Qs},
@@ -106,18 +108,19 @@ accept(Body, ReqData, Context) ->
                fun(E) -> rabbit_mgmt_util:bad_request(E, ReqData, Context) end).
 
 apply_defs(Body, SuccessFun, ErrorFun) ->
-    case rabbit_mgmt_util:decode(
-           [users, vhosts, permissions, queues, exchanges, bindings], Body) of
+    case rabbit_mgmt_util:decode([], Body) of
         {error, E} ->
             ErrorFun(E);
-        {ok, [Users, VHosts, Permissions, Queues, Exchanges, Bindings], _} ->
+        {ok, _, All} ->
             try
-                for_all(Users,       fun add_user/1),
-                for_all(VHosts,      fun add_vhost/1),
-                for_all(Permissions, fun add_permission/1),
-                for_all(Queues,      fun add_queue/1),
-                for_all(Exchanges,   fun add_exchange/1),
-                for_all(Bindings,    fun add_binding/1),
+                for_all(parameters,  All, fun add_parameter/1),
+                for_all(policies,    All, fun add_policy/1),
+                for_all(users,       All, fun add_user/1),
+                for_all(vhosts,      All, fun add_vhost/1),
+                for_all(permissions, All, fun add_permission/1),
+                for_all(queues,      All, fun add_queue/1),
+                for_all(exchanges,   All, fun add_exchange/1),
+                for_all(bindings,    All, fun add_binding/1),
                 SuccessFun()
             catch {error, E} -> ErrorFun(E);
                   exit:E     -> ErrorFun(E)
@@ -154,7 +157,9 @@ export_name(_Name)                -> true.
 %%--------------------------------------------------------------------
 
 rw_state() ->
-    [{users,       [name, password_hash, tags]},
+    [{parameters,  [vhost, component, name, value]},
+     {policies,    [vhost, name, pattern, definition, priority]},
+     {users,       [name, password_hash, tags]},
      {vhosts,      [name]},
      {permissions, [user, vhost, configure, write, read]},
      {queues,      [name, vhost, durable, auto_delete, arguments]},
@@ -174,18 +179,44 @@ filter_item(Item, Allowed) ->
 
 %%--------------------------------------------------------------------
 
-for_all(List, Fun) ->
-    [Fun([{atomise_name(K), clean_value(V)} || {K, V} <- I]) ||
-        {struct, I} <- List].
+for_all(Name, All, Fun) ->
+    case pget(Name, All) of
+        undefined ->
+            ok;
+        List ->
+            [Fun([{atomise_name(K), V} || {K, V} <- I]) ||
+                {struct, I} <- List]
+    end.
 
 atomise_name(N) ->
     list_to_atom(binary_to_list(N)).
 
-clean_value({struct, L}) -> L;
-clean_value(A)           -> A.
-
 %%--------------------------------------------------------------------
 
+add_parameter(Param) ->
+    VHost = pget(vhost, Param),
+    Comp = pget(component, Param),
+    Key = pget(name, Param),
+    case rabbit_runtime_parameters:set(
+           VHost, Comp, Key, rabbit_misc:json_to_term(pget(value, Param))) of
+        ok                -> ok;
+        {error_string, E} -> S = rabbit_misc:format(" (~s/~s/~s)",
+                                                    [VHost, Comp, Key]),
+                             exit(list_to_binary(E ++ S))
+    end.
+
+add_policy(Param) ->
+    VHost = pget(vhost, Param),
+    Key = pget(name, Param),
+    case rabbit_policy:set(
+           VHost, Key, pget(pattern, Param),
+           rabbit_misc:json_to_term(pget(definition, Param)),
+           pget(priority, Param)) of
+        ok                -> ok;
+        {error_string, E} -> S = rabbit_misc:format(" (~s/~s)", [VHost, Key]),
+                             exit(list_to_binary(E ++ S))
+    end.
+
 add_user(User) ->
     rabbit_mgmt_wm_user:put_user(User).
 
