@@ -11,7 +11,7 @@
 %%   The Original Code is RabbitMQ Management Plugin.
 %%
 %%   The Initial Developer of the Original Code is VMware, Inc.
-%%   Copyright (c) 2011-2012 VMware, Inc.  All rights reserved.
+%%   Copyright (c) 2011-2013 VMware, Inc.  All rights reserved.
 %%
 
 -module(rabbit_mgmt_wm_exchange_publish).
@@ -46,7 +46,7 @@ do_it(ReqData, Context) ->
     X = rabbit_mgmt_util:id(exchange, ReqData),
     rabbit_mgmt_util:with_decode(
       [routing_key, properties, payload, payload_encoding], ReqData, Context,
-      fun([RoutingKey, Props0, Payload0, Enc], _) ->
+      fun ([RoutingKey, Props0, Payload0, Enc], _) when is_binary(Payload0) ->
               rabbit_mgmt_util:with_channel(
                 VHost, ReqData, Context,
                 fun (Ch) ->
@@ -73,7 +73,9 @@ do_it(ReqData, Context) ->
                             {'DOWN', _, _, _, Err} ->
                                 bad(Err, ReqData, Context)
                         end
-                end)
+                end);
+          ([_RoutingKey, _Props, _Payload, _Enc], _) ->
+              throw({error, payload_not_string})
       end).
 
 good(MRef, Routed, ReqData, Context) ->
