@@ -11,7 +11,7 @@
 %%   The Original Code is RabbitMQ Management Console.
 %%
 %%   The Initial Developer of the Original Code is VMware, Inc.
-%%   Copyright (c) 2010-2012 VMware, Inc.  All rights reserved.
+%%   Copyright (c) 2010-2013 VMware, Inc.  All rights reserved.
 %%
 
 -module(rabbit_mgmt_wm_node).
@@ -49,5 +49,15 @@ node0(ReqData) ->
     case [N || N <- rabbit_mgmt_wm_nodes:all_nodes(),
                proplists:get_value(name, N) == Name] of
         []     -> not_found;
-        [Node] -> Node
+        [Node] -> augment(ReqData, Name, Node)
+    end.
+
+augment(ReqData, Name, Node) ->
+    case wrq:get_qs_value("memory", ReqData) of
+        "true" -> Mem = case rpc:call(Name, rabbit_vm, memory, [], infinity) of
+                            {badrpc, _} -> not_available;
+                            Memory      -> Memory
+                        end,
+                  [{memory, Mem} | Node];
+        _      -> Node
     end.
