@@ -11,13 +11,15 @@
 %%   The Original Code is RabbitMQ Management Plugin.
 %%
 %%   The Initial Developer of the Original Code is VMware, Inc.
-%%   Copyright (c) 2010-2012 VMware, Inc.  All rights reserved.
+%%   Copyright (c) 2010-2013 VMware, Inc.  All rights reserved.
 %%
 
 -module(rabbit_mgmt_wm_overview).
 
 -export([init/1, to_json/2, content_types_provided/2, is_authorized/2]).
 
+-import(rabbit_misc, [pget/2, pget/3]).
+
 -include("rabbit_mgmt.hrl").
 -include_lib("webmachine/include/webmachine.hrl").
 -include_lib("rabbit_common/include/rabbit.hrl").
@@ -32,11 +34,14 @@ content_types_provided(ReqData, Context) ->
 to_json(ReqData, Context = #context{user = User = #user{tags = Tags}}) ->
     {ok, StatsLevel} = application:get_env(rabbit, collect_statistics),
     %% NB: this duplicates what's in /nodes but we want a global idea
-    %% of this. And /nodes is not accepssible to non-monitor users.
+    %% of this. And /nodes is not accessible to non-monitor users.
     ExchangeTypes = rabbit_mgmt_external_stats:list_registry_plugins(exchange),
-    Overview0 = [{management_version, version()},
+    Overview0 = [{management_version, version(rabbitmq_management)},
                  {statistics_level,   StatsLevel},
-                 {exchange_types,     ExchangeTypes}],
+                 {exchange_types,     ExchangeTypes},
+                 {rabbitmq_version,   version(rabbit)},
+                 {erlang_version,     list_to_binary(
+                                        erlang:system_info(otp_release))}],
     Overview =
         case rabbit_mgmt_util:is_monitor(Tags) of
             true ->
@@ -63,10 +68,9 @@ stats_db_node() ->
         Pid       -> node(Pid)
     end.
 
-version() ->
-    [Vsn] = [V || {A, _D, V} <- application:loaded_applications(),
-            A =:= rabbitmq_management],
-    list_to_binary(Vsn).
+version(App) ->
+    {_, _, V} = lists:keyfind(App, 1, application:which_applications()),
+    list_to_binary(V).
 
 listeners() ->
     rabbit_mgmt_util:sort_list(
@@ -78,29 +82,9 @@ listeners() ->
 
 rabbit_mochiweb_contexts() ->
     rabbit_mgmt_util:sort_list(
-      lists:append([contexts(Node) ||
-                       Node <- rabbit_mnesia:running_clustered_nodes()]),
+      lists:append(
+        [rabbit_mochiweb_contexts(N) || N <- rabbit_mgmt_wm_nodes:all_nodes()]),
       ["description", "port", "node"]).
 
-contexts(Node) ->
-    case rpc:call(Node, rabbit_mochiweb_registry, list_all, [], infinity) of
-        {badrpc, {'EXIT', _}} ->
-            [];
-        Contexts ->
-            [[{node, Node} | format_context(C)] || C <- Contexts]
-    end.
-
-format_context({Path, Description, Rest}) ->
-    [{description, list_to_binary(Description)},
-     {path,        list_to_binary("/" ++ Path)} |
-     format_mochiweb_option_list(Rest)].
-
-format_mochiweb_option_list(C) ->
-    [{K, format_mochiweb_option(K, V)} || {K, V} <- C].
-
-format_mochiweb_option(ssl_opts, V) ->
-    format_mochiweb_option_list(V);
-format_mochiweb_option(_K, V) when is_list(V) ->
-    list_to_binary(V);
-format_mochiweb_option(_K, V) ->
-    V.
+rabbit_mochiweb_contexts(N) ->
+    [[{node, pget(name, N)} | C] || C <- pget(contexts, N, [])].
