@@ -11,7 +11,7 @@
 %%   The Original Code is RabbitMQ Management Console.
 %%
 %%   The Initial Developer of the Original Code is VMware, Inc.
-%%   Copyright (c) 2010-2012 VMware, Inc.  All rights reserved.
+%%   Copyright (c) 2010-2013 VMware, Inc.  All rights reserved.
 %%
 
 -module(rabbit_mgmt_test_all).
@@ -20,7 +20,7 @@
 
 all_tests() ->
     ok = eunit:test(rabbit_mgmt_test_unit,[verbose]),
-    ok = eunit:test(rabbit_mgmt_test_http,[verbose]),
+    ok = eunit:test(tests(rabbit_mgmt_test_http, 60), [verbose]),
     io:format("Starting second node...~n"),
     ok = rabbit_mgmt_test_clustering:start_second_node(),
     io:format("...done.~n"),
@@ -30,3 +30,10 @@ all_tests() ->
         ok = rabbit_mgmt_test_clustering:stop_second_node()
     end,
     ok = rabbit_mgmt_test_db:test().
+
+
+tests(Module, Timeout) ->
+    {foreach, fun() -> ok end,
+     [{timeout, Timeout, fun Module:F/0} ||
+         {F, _Arity} <- proplists:get_value(exports, Module:module_info()),
+         string:right(atom_to_list(F), 5) =:= "_test"]}.
