@@ -11,7 +11,7 @@
 %%   The Original Code is RabbitMQ Management Console.
 %%
 %%   The Initial Developer of the Original Code is VMware, Inc.
-%%   Copyright (c) 2010-2012 VMware, Inc.  All rights reserved.
+%%   Copyright (c) 2010-2013 VMware, Inc.  All rights reserved.
 %%
 
 -module(rabbit_mgmt_test_db).
@@ -262,7 +262,7 @@ find_conn_by_local_port(Port, Items) ->
     [Conn] = lists:filter(
                fun(Conn) ->
                        pget(peer_port, Conn) == Port andalso
-                           pget(peer_address, Conn) == <<"127.0.0.1">>
+                           pget(peer_host, Conn) == <<"127.0.0.1">>
                end, Items),
     Conn.
 
