@@ -11,7 +11,7 @@
 %%   The Original Code is RabbitMQ Management Console.
 %%
 %%   The Initial Developer of the Original Code is VMware, Inc.
-%%   Copyright (c) 2010-2012 VMware, Inc.  All rights reserved.
+%%   Copyright (c) 2010-2013 VMware, Inc.  All rights reserved.
 %%
 
 -module(rabbit_mgmt_test_http).
@@ -285,10 +285,8 @@ bindings_test() ->
     QArgs = [],
     http_put("/exchanges/%2f/myexchange", XArgs, ?NO_CONTENT),
     http_put("/queues/%2f/myqueue", QArgs, ?NO_CONTENT),
-    http_put("/bindings/%2f/e/myexchange/q/badqueue/routing", [], ?NOT_FOUND),
-    http_put("/bindings/%2f/e/badexchange/q/myqueue/routing", [], ?NOT_FOUND),
-    http_put("/bindings/%2f/e/myexchange/q/myqueue/bad_routing", [], ?BAD_REQUEST),
-    http_put("/bindings/%2f/e/myexchange/q/myqueue/routing", [], ?NO_CONTENT),
+    BArgs = [{routing_key, <<"routing">>}, {arguments, []}],
+    http_post("/bindings/%2f/e/myexchange/q/myqueue", BArgs, ?CREATED),
     http_get("/bindings/%2f/e/myexchange/q/myqueue/routing", ?OK),
     http_get("/bindings/%2f/e/myexchange/q/myqueue/rooting", ?NOT_FOUND),
     Binding =
@@ -332,19 +330,22 @@ bindings_post_test() ->
     http_post("/bindings/%2f/e/myexchange/q/badqueue", BArgs, ?NOT_FOUND),
     http_post("/bindings/%2f/e/badexchange/q/myqueue", BArgs, ?NOT_FOUND),
     Headers1 = http_post("/bindings/%2f/e/myexchange/q/myqueue", [], ?CREATED),
-    "../../../../%2F/e/myexchange/q/myqueue/_" = pget("location", Headers1),
+    "../../../../%2F/e/myexchange/q/myqueue/~" = pget("location", Headers1),
     Headers2 = http_post("/bindings/%2f/e/myexchange/q/myqueue", BArgs, ?CREATED),
-    "../../../../%2F/e/myexchange/q/myqueue/routing_foo_bar" =
+    PropertiesKey = "routing~V4mGFgnPNrdtRmluZIxTDA",
+    PropertiesKeyBin = list_to_binary(PropertiesKey),
+    "../../../../%2F/e/myexchange/q/myqueue/" ++ PropertiesKey =
         pget("location", Headers2),
+    URI = "/bindings/%2F/e/myexchange/q/myqueue/" ++ PropertiesKey,
     [{source,<<"myexchange">>},
      {vhost,<<"/">>},
      {destination,<<"myqueue">>},
      {destination_type,<<"queue">>},
      {routing_key,<<"routing">>},
      {arguments,[{foo,<<"bar">>}]},
-     {properties_key,<<"routing_foo_bar">>}] =
-        http_get("/bindings/%2F/e/myexchange/q/myqueue/routing_foo_bar", ?OK),
-    http_delete("/bindings/%2F/e/myexchange/q/myqueue/routing_foo_bar", ?NO_CONTENT),
+     {properties_key,PropertiesKeyBin}] = http_get(URI, ?OK),
+    http_get(URI ++ "x", ?NOT_FOUND),
+    http_delete(URI, ?NO_CONTENT),
     http_delete("/exchanges/%2f/myexchange", ?NO_CONTENT),
     http_delete("/queues/%2f/myqueue", ?NO_CONTENT),
     ok.
@@ -365,7 +366,7 @@ bindings_e2e_test() ->
      {properties_key,<<"routing">>}] =
         http_get("/bindings/%2f/e/amq.direct/e/amq.fanout/routing", ?OK),
     http_delete("/bindings/%2f/e/amq.direct/e/amq.fanout/routing", ?NO_CONTENT),
-    http_put("/bindings/%2f/e/amq.direct/e/amq.headers/routing", [], ?NO_CONTENT),
+    http_post("/bindings/%2f/e/amq.direct/e/amq.headers", BArgs, ?CREATED),
     Binding =
         [{source,<<"amq.direct">>},
          {vhost,<<"/">>},
@@ -483,15 +484,16 @@ get_conn(Username, Password) ->
                                         username = list_to_binary(Username),
                                         password = list_to_binary(Password)}),
     LocalPort = rabbit_mgmt_test_db:local_port(Conn),
-    ConnPath = binary_to_list(
-                 rabbit_mgmt_format:print(
-                   "/connections/127.0.0.1%3A~w%20->%20127.0.0.1%3A5672",
-                   [LocalPort])),
-    ChPath = binary_to_list(
-               rabbit_mgmt_format:print(
-                 "/channels/127.0.0.1%3A~w%20->%20127.0.0.1%3A5672%20(1)",
-                 [LocalPort])),
-    {Conn, ConnPath, ChPath}.
+    ConnPath = rabbit_misc:format(
+                 "/connections/127.0.0.1%3A~w%20->%20127.0.0.1%3A5672",
+                 [LocalPort]),
+    ChPath = rabbit_misc:format(
+               "/channels/127.0.0.1%3A~w%20->%20127.0.0.1%3A5672%20(1)",
+               [LocalPort]),
+    ConnChPath = rabbit_misc:format(
+                   "/connections/127.0.0.1%3A~w%20->%20127.0.0.1%3A5672/channels",
+                   [LocalPort]),
+    {Conn, ConnPath, ChPath, ConnChPath}.
 
 permissions_connection_channel_test() ->
     PermArgs = [{configure, <<".*">>}, {write, <<".*">>}, {read, <<".*">>}],
@@ -501,9 +503,9 @@ permissions_connection_channel_test() ->
     http_put("/users/monitor", [{password, <<"monitor">>},
                             {tags, <<"monitoring">>}], ?NO_CONTENT),
     http_put("/permissions/%2f/monitor", PermArgs, ?NO_CONTENT),
-    {Conn1, UserConn, UserCh} = get_conn("user", "user"),
-    {Conn2, MonConn, MonCh} = get_conn("monitor", "monitor"),
-    {Conn3, AdmConn, AdmCh} = get_conn("guest", "guest"),
+    {Conn1, UserConn, UserCh, UserConnCh} = get_conn("user", "user"),
+    {Conn2, MonConn, MonCh, MonConnCh} = get_conn("monitor", "monitor"),
+    {Conn3, AdmConn, AdmCh, AdmConnCh} = get_conn("guest", "guest"),
     {ok, _Ch1} = amqp_connection:open_channel(Conn1),
     {ok, _Ch2} = amqp_connection:open_channel(Conn2),
     {ok, _Ch3} = amqp_connection:open_channel(Conn3),
@@ -529,6 +531,9 @@ permissions_connection_channel_test() ->
     AssertRead(UserCh, ?OK),
     AssertRead(MonCh, ?NOT_AUTHORISED),
     AssertRead(AdmCh, ?NOT_AUTHORISED),
+    AssertRead(UserConnCh, ?OK),
+    AssertRead(MonConnCh, ?NOT_AUTHORISED),
+    AssertRead(AdmConnCh, ?NOT_AUTHORISED),
 
     AssertClose = fun(Path, User, Status) ->
                           http_delete(Path, User, User, Status)
@@ -552,38 +557,73 @@ unicode_test() ->
     http_delete("/queues/%2f/♫♪♫♪", ?NO_CONTENT),
     ok.
 
-definitions_test() ->
-    XArgs = [{type, <<"direct">>}],
-    QArgs = [],
-    http_put("/queues/%2f/my-queue", QArgs, ?NO_CONTENT),
-    http_put("/exchanges/%2f/my-exchange", XArgs, ?NO_CONTENT),
-    http_put("/bindings/%2f/e/my-exchange/q/my-queue/routing", [], ?NO_CONTENT),
-    http_put("/bindings/%2f/e/amq.direct/q/my-queue/routing", [], ?NO_CONTENT),
-    http_put("/bindings/%2f/e/amq.direct/e/amq.fanout/routing", [], ?NO_CONTENT),
+defs(Key, URI, CreateMethod, Args) ->
+    %% Create the item
+    URI2 = case CreateMethod of
+               put   -> http_put(URI, Args, ?NO_CONTENT),
+                        URI;
+               post  -> Headers = http_post(URI, Args, ?CREATED),
+                        "../../../.." ++ Loc = pget("location", Headers),
+                        "/" ++ atom_to_list(Key) ++ Loc
+           end,
+    %% Make sure it ends up in definitions
     Definitions = http_get("/definitions", ?OK),
-    http_delete("/bindings/%2f/e/my-exchange/q/my-queue/routing", ?NO_CONTENT),
-    http_delete("/bindings/%2f/e/amq.direct/q/my-queue/routing", ?NO_CONTENT),
-    http_delete("/bindings/%2f/e/amq.direct/e/amq.fanout/routing", ?NO_CONTENT),
-    http_delete("/queues/%2f/my-queue", ?NO_CONTENT),
-    http_delete("/exchanges/%2f/my-exchange", ?NO_CONTENT),
-    http_post("/definitions", Definitions, ?NO_CONTENT),
-    http_delete("/bindings/%2f/e/my-exchange/q/my-queue/routing", ?NO_CONTENT),
-    http_delete("/bindings/%2f/e/amq.direct/q/my-queue/routing", ?NO_CONTENT),
-    http_delete("/bindings/%2f/e/amq.direct/e/amq.fanout/routing", ?NO_CONTENT),
-    http_delete("/queues/%2f/my-queue", ?NO_CONTENT),
-    http_delete("/exchanges/%2f/my-exchange", ?NO_CONTENT),
-    ExtraConfig =
-        [{users,       []},
-         {vhosts,      []},
-         {permissions, []},
-         {queues,       [[{name,        <<"another-queue">>},
-                          {vhost,       <<"/">>},
-                          {durable,     true},
-                          {auto_delete, false},
-                          {arguments,   []}
-                         ]]},
-         {exchanges,   []},
-         {bindings,    []}],
+    true = lists:any(fun(I) -> test_item(Args, I) end, pget(Key, Definitions)),
+
+    %% Delete it
+    http_delete(URI2, ?NO_CONTENT),
+
+    %% Post the definitions back, it should get recreated in correct form
+    http_post("/definitions", Definitions, ?CREATED),
+    assert_item(Args, http_get(URI2, ?OK)),
+
+    %% And delete it again
+    http_delete(URI2, ?NO_CONTENT),
+
+    ok.
+
+definitions_test() ->
+    rabbit_runtime_parameters_test:register(),
+    rabbit_runtime_parameters_test:register_policy_validator(),
+
+    defs(queues, "/queues/%2f/my-queue", put,
+         [{name,    <<"my-queue">>},
+          {durable, true}]),
+    defs(exchanges, "/exchanges/%2f/my-exchange", put,
+         [{name, <<"my-exchange">>},
+          {type, <<"direct">>}]),
+    defs(bindings, "/bindings/%2f/e/amq.direct/e/amq.fanout", post,
+         [{routing_key, <<"routing">>}, {arguments, []}]),
+    defs(policies, "/policies/%2f/my-policy", put,
+         [{vhost,      <<"/">>},
+          {name,       <<"my-policy">>},
+          {pattern,    <<".*">>},
+          {definition, [{testpos, [1, 2, 3]}]},
+          {priority,   1}]),
+    defs(parameters, "/parameters/test/%2f/good", put,
+         [{vhost,     <<"/">>},
+          {component, <<"test">>},
+          {name,      <<"good">>},
+          {value,     <<"ignore">>}]),
+    defs(users, "/users/myuser", put,
+         [{name,          <<"myuser">>},
+          {password_hash, <<"WAbU0ZIcvjTpxM3Q3SbJhEAM2tQ=">>},
+          {tags,          <<"management">>}]),
+    defs(vhosts, "/vhosts/myvhost", put,
+         [{name, <<"myvhost">>}]),
+    defs(permissions, "/permissions/%2f/guest", put,
+         [{user,      <<"guest">>},
+          {vhost,     <<"/">>},
+          {configure, <<"c">>},
+          {write,     <<"w">>},
+          {read,      <<"r">>}]),
+
+    %% We just messed with guest's permissions
+    http_put("/permissions/%2f/guest",
+             [{configure, <<".*">>},
+              {write,     <<".*">>},
+              {read,      <<".*">>}], ?NO_CONTENT),
+
     BrokenConfig =
         [{users,       []},
          {vhosts,      []},
@@ -597,9 +637,10 @@ definitions_test() ->
                          {arguments,   []}
                         ]]},
          {bindings,    []}],
-    http_post("/definitions", ExtraConfig, ?NO_CONTENT),
     http_post("/definitions", BrokenConfig, ?BAD_REQUEST),
-    http_delete("/queues/%2f/another-queue", ?NO_CONTENT),
+
+    rabbit_runtime_parameters_test:unregister_policy_validator(),
+    rabbit_runtime_parameters_test:unregister(),
     ok.
 
 definitions_remove_things_test() ->
@@ -628,7 +669,7 @@ definitions_server_named_queue_test() ->
     Definitions = http_get("/definitions", ?OK),
     http_delete(Path, ?NO_CONTENT),
     http_get(Path, ?NOT_FOUND),
-    http_post("/definitions", Definitions, ?NO_CONTENT),
+    http_post("/definitions", Definitions, ?CREATED),
     http_get(Path, ?OK),
     http_delete(Path, ?NO_CONTENT),
     ok.
@@ -652,15 +693,15 @@ arguments_test() ->
     Definitions = http_get("/definitions", ?OK),
     http_delete("/exchanges/%2f/myexchange", ?NO_CONTENT),
     http_delete("/queues/%2f/myqueue", ?NO_CONTENT),
-    http_post("/definitions", Definitions, ?NO_CONTENT),
+    http_post("/definitions", Definitions, ?CREATED),
     [{'alternate-exchange', <<"amq.direct">>}] =
         pget(arguments, http_get("/exchanges/%2f/myexchange", ?OK)),
     [{'x-expires', 1800000}] =
         pget(arguments, http_get("/queues/%2f/myqueue", ?OK)),
-    [{foo, <<"bar">>}, {'x-match', <<"all">>}] =
-        pget(arguments,
-             http_get("/bindings/%2f/e/myexchange/q/myqueue/" ++
-                          "_foo_bar_x-match_all", ?OK)),
+    true = lists:sort([{'x-match', <<"all">>}, {foo, <<"bar">>}]) =:=
+           lists:sort(pget(arguments,
+                           http_get("/bindings/%2f/e/myexchange/q/myqueue/" ++
+                                    "~nXOkVwqZzUOdS9_HcBWheg", ?OK))),
     http_delete("/exchanges/%2f/myexchange", ?NO_CONTENT),
     http_delete("/queues/%2f/myqueue", ?NO_CONTENT),
     ok.
@@ -673,7 +714,7 @@ arguments_table_test() ->
     http_put("/exchanges/%2f/myexchange", XArgs, ?NO_CONTENT),
     Definitions = http_get("/definitions", ?OK),
     http_delete("/exchanges/%2f/myexchange", ?NO_CONTENT),
-    http_post("/definitions", Definitions, ?NO_CONTENT),
+    http_post("/definitions", Definitions, ?CREATED),
     Args = pget(arguments, http_get("/exchanges/%2f/myexchange", ?OK)),
     http_delete("/exchanges/%2f/myexchange", ?NO_CONTENT),
     ok.
@@ -763,11 +804,10 @@ sorting_test() ->
     ok.
 
 columns_test() ->
-    http_put("/queues/%2f/test", [], ?NO_CONTENT),
-    %% Bit lame to test backing_queue_status but at least it's
-    %% something we can descend to that's always there
-    [[{backing_queue_status, [{len, 0}]}, {name, <<"test">>}]] =
-        http_get("/queues?columns=backing_queue_status.len,name", ?OK),
+    http_put("/queues/%2f/test", [{arguments, [{<<"foo">>, <<"bar">>}]}],
+             ?NO_CONTENT),
+    [[{name, <<"test">>}, {arguments, [{foo, <<"bar">>}]}]] =
+        http_get("/queues?columns=arguments.foo,name", ?OK),
     http_delete("/queues/%2f/test", ?NO_CONTENT),
     ok.
 
@@ -860,6 +900,12 @@ publish_fail_test() ->
             {payload,          <<"Hello world">>},
             {payload_encoding, <<"string">>}],
     http_post("/exchanges/%2f/amq.default/publish", Msg2, ?BAD_REQUEST),
+    Msg3 = [{exchange,         <<"">>},
+            {routing_key,      <<"myqueue">>},
+            {properties,       []},
+            {payload,          [<<"not a string">>]},
+            {payload_encoding, <<"string">>}],
+    http_post("/exchanges/%2f/amq.default/publish", Msg3, ?BAD_REQUEST),
     http_delete("/users/myuser", ?NO_CONTENT),
     ok.
 
@@ -883,6 +929,81 @@ publish_unrouted_test() ->
     ?assertEqual([{routed, false}],
                  http_post("/exchanges/%2f/amq.default/publish", Msg, ?OK)).
 
+parameters_test() ->
+    rabbit_runtime_parameters_test:register(),
+
+    http_put("/parameters/test/%2f/good", [{value, <<"ignore">>}], ?NO_CONTENT),
+    http_put("/parameters/test/%2f/maybe", [{value, <<"good">>}], ?NO_CONTENT),
+    http_put("/parameters/test/%2f/maybe", [{value, <<"bad">>}], ?BAD_REQUEST),
+    http_put("/parameters/test/%2f/bad", [{value, <<"good">>}], ?BAD_REQUEST),
+    http_put("/parameters/test/um/good", [{value, <<"ignore">>}], ?NOT_FOUND),
+
+    Good = [{vhost,     <<"/">>},
+            {component, <<"test">>},
+            {name,      <<"good">>},
+            {value,     <<"ignore">>}],
+    Maybe = [{vhost,     <<"/">>},
+             {component, <<"test">>},
+             {name,      <<"maybe">>},
+             {value,     <<"good">>}],
+    List = [Good, Maybe],
+
+    assert_list(List, http_get("/parameters")),
+    assert_list(List, http_get("/parameters/test")),
+    assert_list(List, http_get("/parameters/test/%2f")),
+    http_get("/parameters/oops", ?NOT_FOUND),
+    http_get("/parameters/test/oops", ?NOT_FOUND),
+
+    assert_item(Good,  http_get("/parameters/test/%2f/good", ?OK)),
+    assert_item(Maybe, http_get("/parameters/test/%2f/maybe", ?OK)),
+
+    http_delete("/parameters/test/%2f/good", ?NO_CONTENT),
+    http_delete("/parameters/test/%2f/maybe", ?NO_CONTENT),
+    http_delete("/parameters/test/%2f/bad", ?NOT_FOUND),
+
+    0 = length(http_get("/parameters")),
+    0 = length(http_get("/parameters/test")),
+    0 = length(http_get("/parameters/test/%2f")),
+    rabbit_runtime_parameters_test:unregister(),
+    ok.
+
+policy_test() ->
+    rabbit_runtime_parameters_test:register_policy_validator(),
+    PolicyPos  = [{vhost,      <<"/">>},
+                  {name,       <<"policy_pos">>},
+                  {pattern,    <<".*">>},
+                  {definition, [{testpos,[1,2,3]}]},
+                  {priority,   10}],
+    PolicyEven = [{vhost,      <<"/">>},
+                  {name,       <<"policy_even">>},
+                  {pattern,    <<".*">>},
+                  {definition, [{testeven,[1,2,3,4]}]},
+                  {priority,   10}],
+    http_put(
+      "/policies/%2f/policy_pos",
+      lists:keydelete(key, 1, PolicyPos),
+      ?NO_CONTENT),
+    http_put(
+      "/policies/%2f/policy_even",
+      lists:keydelete(key, 1, PolicyEven),
+      ?NO_CONTENT),
+    assert_item(PolicyPos,  http_get("/policies/%2f/policy_pos",  ?OK)),
+    assert_item(PolicyEven, http_get("/policies/%2f/policy_even", ?OK)),
+    List = [PolicyPos, PolicyEven],
+    assert_list(List, http_get("/policies",     ?OK)),
+    assert_list(List, http_get("/policies/%2f", ?OK)),
+
+    http_delete("/policies/%2f/policy_pos", ?NO_CONTENT),
+    http_delete("/policies/%2f/policy_even", ?NO_CONTENT),
+    0 = length(http_get("/policies")),
+    0 = length(http_get("/policies/%2f")),
+    rabbit_runtime_parameters_test:unregister_policy_validator(),
+    ok.
+
+extensions_test() ->
+    [[{javascript,<<"dispatcher.js">>}]] = http_get("/extensions", ?OK),
+    ok.
+
 %%---------------------------------------------------------------------------
 
 msg(Key, Headers, Body) ->
