@@ -11,7 +11,7 @@
 %%   The Original Code is RabbitMQ Management Console.
 %%
 %%   The Initial Developer of the Original Code is VMware, Inc.
-%%   Copyright (c) 2010-2012 VMware, Inc.  All rights reserved.
+%%   Copyright (c) 2010-2013 VMware, Inc.  All rights reserved.
 %%
 
 -module(rabbit_mgmt_test_unit).
@@ -21,29 +21,19 @@
 tokenise_test() ->
     [] = rabbit_mgmt_format:tokenise(""),
     ["foo"] = rabbit_mgmt_format:tokenise("foo"),
-    ["foo", "bar"] = rabbit_mgmt_format:tokenise("foo_bar"),
-    ["foo", "", "bar"] = rabbit_mgmt_format:tokenise("foo__bar"),
+    ["foo", "bar"] = rabbit_mgmt_format:tokenise("foo~bar"),
+    ["foo", "", "bar"] = rabbit_mgmt_format:tokenise("foo~~bar"),
     ok.
 
 pack_binding_test() ->
-    assert_binding(<<"_">>,
+    assert_binding(<<"~">>,
                    <<"">>, []),
     assert_binding(<<"foo">>,
                    <<"foo">>, []),
-    assert_binding(<<"foo%5Fbar%2Fbash">>,
-                   <<"foo_bar/bash">>, []),
-    assert_binding(<<"foo%5Fbar%5Fbash">>,
-                   <<"foo_bar_bash">>, []),
-    assert_binding(<<"foo_arg1_foo%5Fbar%2Fbash">>,
-                   <<"foo">>, [{<<"arg1">>, longstr, <<"foo_bar/bash">>}]),
-    assert_binding(<<"foo_arg1_bar_arg2_bash">>,
-                   <<"foo">>, [{<<"arg1">>, longstr, <<"bar">>},
-                               {<<"arg2">>, longstr, <<"bash">>}]),
-    assert_binding(<<"_arg1_bar">>,
-                   <<"">>, [{<<"arg1">>, longstr, <<"bar">>}]),
-
-    {bad_request, {no_value, "routing"}} =
-        rabbit_mgmt_format:unpack_binding_props(<<"bad_routing">>),
+    assert_binding(<<"foo%7Ebar%2Fbash">>,
+                   <<"foo~bar/bash">>, []),
+    assert_binding(<<"foo%7Ebar%7Ebash">>,
+                   <<"foo~bar~bash">>, []),
     ok.
 
 amqp_table_test() ->
@@ -70,10 +60,4 @@ assert_binding(Packed, Routing, Args) ->
             ok;
         Act ->
             throw({pack, Routing, Args, expected, Packed, got, Act})
-    end,
-    case rabbit_mgmt_format:unpack_binding_props(Packed) of
-        {Routing, Args} ->
-            ok;
-        Act2 ->
-            throw({unpack, Packed, expected, Routing, Args, got, Act2})
     end.
