@@ -26,8 +26,13 @@ class TestRabbitMQAdmin(unittest.TestCase):
         self.run_fail(['--host', 'some-host-that-does-not-exist', 'show', 'overview'])
 
     def test_port(self):
+        # Test port selection
+        self.run_success(['--port', '15672', 'show', 'overview'])
+        # Test redirect
         self.run_success(['--port', '55672', 'show', 'overview'])
-        self.run_fail(['--port', '55673', 'show', 'overview'])
+        # Test port not open
+        self.run_fail(['--port', '15673', 'show', 'overview'])
+        # Test port open but not talking HTTP
         self.run_fail(['--port', '5672', 'show', 'overview'])
 
     def test_user(self):
@@ -119,6 +124,18 @@ tracing: False
         self.assert_table([['', 'foo', 'queue', 'foo'], ['amq.direct', 'foo', 'queue', 'test']], ['list', 'bindings', 'source', 'destination', 'destination_type', 'routing_key'])
         self.run_success(['delete', 'queue', 'name=foo'])
 
+    def test_policies(self):
+        self.run_success(['declare', 'policy', 'name=ha', 'pattern=.*', 'definition={"ha-mode":"all"}'])
+        self.assert_table([['ha', '/', '.*', '{"ha-mode": "all"}']], ['list', 'policies', 'name', 'vhost', 'pattern', 'definition'])
+        self.run_success(['delete', 'policy', 'name=ha'])
+
+    def test_parameters(self):
+        self.ctl(['eval', 'rabbit_runtime_parameters_test:register().'])
+        self.run_success(['declare', 'parameter', 'component=test', 'name=good', 'value=123'])
+        self.assert_table([['test', 'good', '/', '123']], ['list', 'parameters', 'component', 'name', 'vhost', 'value'])
+        self.run_success(['delete', 'parameter', 'component=test', 'name=good'])
+        self.ctl(['eval', 'rabbit_runtime_parameters_test:unregister().'])
+
     def test_publish(self):
         self.run_success(['declare', 'queue', 'name=test'])
         self.run_success(['publish', 'routing_key=test', 'payload=test_1'])
@@ -145,35 +162,58 @@ tracing: False
         self.run_success(['--vhost', '/', 'declare', 'user', 'name=foo', 'password=pass', 'tags='])
         self.run_success(['delete', 'user', 'name=foo'])
 
+    def test_sort(self):
+        self.run_success(['declare', 'queue', 'name=foo'])
+        self.run_success(['declare', 'binding', 'source=amq.direct', 'destination=foo', 'destination_type=queue', 'routing_key=bbb'])
+        self.run_success(['declare', 'binding', 'source=amq.topic', 'destination=foo', 'destination_type=queue', 'routing_key=aaa'])
+        self.assert_table([['', 'foo'], ['amq.direct', 'bbb'], ['amq.topic', 'aaa']], ['--sort', 'source', 'list', 'bindings', 'source', 'routing_key'])
+        self.assert_table([['amq.topic', 'aaa'], ['amq.direct', 'bbb'], ['', 'foo']], ['--sort', 'routing_key', 'list', 'bindings', 'source', 'routing_key'])
+        self.assert_table([['amq.topic', 'aaa'], ['amq.direct', 'bbb'], ['', 'foo']], ['--sort', 'source', '--sort-reverse', 'list', 'bindings', 'source', 'routing_key'])
+        self.run_success(['delete', 'queue', 'name=foo'])
+
     # ---------------------------------------------------------------------------
 
     def run_success(self, args, **kwargs):
-        self.assertEqual(0, run(args, **kwargs)[1])
+        (stdout, ret) = self.admin(args, **kwargs)
+        if ret != 0:
+            self.fail(stdout)
 
     def run_fail(self, args):
-        self.assertNotEqual(0, run(args)[1])
+        (stdout, ret) = self.admin(args)
+        if ret == 0:
+            self.fail(stdout)
 
     def assert_output(self, expected, args):
-        self.assertEqual(expected, run(args)[0])
+        self.assertEqual(expected, self.admin(args)[0])
 
     def assert_list(self, expected, args0):
         args = ['-f', 'tsv', '-q']
         args.extend(args0)
-        self.assertEqual(expected, run(args)[0].splitlines())
+        self.assertEqual(expected, self.admin(args)[0].splitlines())
 
     def assert_table(self, expected, args0):
         args = ['-f', 'tsv', '-q']
         args.extend(args0)
-        self.assertEqual(expected, [l.split('\t') for l in run(args)[0].splitlines()])
+        self.assertEqual(expected, [l.split('\t') for l in self.admin(args)[0].splitlines()])
 
-def run(args, stdin=None):
-    path = os.path.normpath(os.path.join(os.getcwd(), sys.argv[0], '../../../bin/rabbitmqadmin'))
+    def admin(self, args, stdin=None):
+        return run('../../../bin/rabbitmqadmin', args, stdin)
+
+    def ctl(self, args0, stdin=None):
+        args = ['-n', 'rabbit-test']
+        args.extend(args0)
+        (stdout, ret) = run('../../../../rabbitmq-server/scripts/rabbitmqctl', args, stdin)
+        if ret != 0:
+            self.fail(stdout)
+
+def run(cmd, args, stdin):
+    path = os.path.normpath(os.path.join(os.getcwd(), sys.argv[0], cmd))
     cmdline = [path]
     cmdline.extend(args)
     proc = subprocess.Popen(cmdline, stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
     (stdout, stderr) = proc.communicate(stdin)
     returncode = proc.returncode
-    return (stdout, returncode)
+    return (stdout + stderr, returncode)
 
 def l(thing):
     return ['list', thing, 'name']
