@@ -1,7 +1,7 @@
 %% This file contains an adapted version of webmachine_mochiweb:loop/1
 %% from webmachine (revision 0c4b60ac68b4).
 
-%% All modifications are (C) 2011-2012 VMware, Inc.
+%% All modifications are (C) 2011-2013 VMware, Inc.
 
 -module(rabbit_webmachine).
 
@@ -42,16 +42,13 @@ setup(LogModule, ErrorHandler) ->
     end.
 
 makeloop(Dispatch) ->
-    fun ({Prefix, _Listener}, MochiReq) ->
+    fun (MochiReq) ->
             Req = webmachine:new_request(mochiweb, MochiReq),
-            {Path0, _} = Req:path(),
-            Path = case string:substr(Path0, length(Prefix) + 2) of
-                       "" -> "/"; %% webmachine requires a leading "/"
-                       P  -> P    %% implicit "/"
-                   end,
-            %% webmachine_mochiweb:loop/1 uses dispatch/3 here;
+            {Path, _} = Req:path(),
+            {ReqData, _} = Req:get_reqdata(),
+            %% webmachine_mochiweb:loop/1 uses dispatch/4 here;
             %% however, we don't need to dispatch by the host name.
-            case webmachine_dispatcher:dispatch(Path, Dispatch) of
+            case webmachine_dispatcher:dispatch(Path, Dispatch, ReqData) of
                 {no_dispatch_match, _Host, _PathElements} ->
                     {ErrorHTML, ReqState1} =
                         webmachine_error_handler:render_error(
