@@ -11,7 +11,7 @@
 %% The Original Code is RabbitMQ.
 %%
 %% The Initial Developer of the Original Code is VMware, Inc.
-%% Copyright (c) 2010-2012 VMware, Inc.  All rights reserved.
+%% Copyright (c) 2010-2013 VMware, Inc.  All rights reserved.
 %%
 
 -module(rabbit_mochiweb_test).
@@ -20,17 +20,18 @@
 
 query_static_resource_test() ->
     %% TODO this is a fairly rubbish test, but not as bad as it was
-    rabbit_mochiweb:register_static_context(test, "rabbit_mochiweb_test",
+    rabbit_mochiweb:register_static_context(test, [{port, 12345}],
+                                            "rabbit_mochiweb_test",
                                             ?MODULE, "priv/www", "Test"),
     {ok, {_Status, _Headers, Body}} =
-        httpc:request("http://localhost:55670/rabbit_mochiweb_test/index.html"),
+        httpc:request("http://localhost:12345/rabbit_mochiweb_test/index.html"),
     ?assert(string:str(Body, "RabbitMQ HTTP Server Test Page") /= 0).
 
 add_idempotence_test() ->
     F = fun(_Req) -> ok end,
     L = {"/foo", "Foo"},
-    rabbit_mochiweb_registry:add(foo, F, F, L),
-    rabbit_mochiweb_registry:add(foo, F, F, L),
+    rabbit_mochiweb_registry:add(foo, [{port, 12345}], F, F, L),
+    rabbit_mochiweb_registry:add(foo, [{port, 12345}], F, F, L),
     ?assertEqual(
        1, length([ok || {"/foo", _, _} <-
                             rabbit_mochiweb_registry:list_all()])),
