@@ -11,7 +11,7 @@
 %%  The Original Code is RabbitMQ.
 %%
 %%  The Initial Developer of the Original Code is VMware, Inc.
-%%  Copyright (c) 2007-2012 VMware, Inc.  All rights reserved.
+%%  Copyright (c) 2007-2013 VMware, Inc.  All rights reserved.
 %%
 
 -module(rabbit_shovel_sup).
@@ -47,7 +47,7 @@ make_child_specs(Configurations) ->
                 {rabbit_shovel_worker, start_link, [ShovelName, ShovelConfig]},
                 case ShovelConfig#shovel.reconnect_delay of
                     0 -> temporary;
-                    N -> {transient, N}
+                    N -> {permanent, N}
                 end,
                 16#ffffffff,
                 worker,
