@@ -11,14 +11,12 @@
 %%  The Original Code is RabbitMQ.
 %%
 %%  The Initial Developer of the Original Code is VMware, Inc.
-%%  Copyright (c) 2007-2012 VMware, Inc.  All rights reserved.
+%%  Copyright (c) 2007-2013 VMware, Inc.  All rights reserved.
 %%
 
 -module(rabbit_shovel_worker).
 -behaviour(gen_server2).
 
--compile({parse_transform, cut}).
-
 -export([start_link/2]).
 -export([init/1, handle_call/3, handle_cast/2, handle_info/2, terminate/2,
          code_change/3]).
@@ -26,6 +24,8 @@
 -include_lib("amqp_client/include/amqp_client.hrl").
 -include("rabbit_shovel.hrl").
 
+-define(MAX_CONNECTION_CLOSE_TIMEOUT, 10000).
+
 -record(state, {inbound_conn, inbound_ch, outbound_conn, outbound_ch,
                 name, config, blocked, msg_buf, inbound_params,
                 outbound_params, unacked}).
@@ -126,13 +126,22 @@ handle_info(#'channel.flow'{active = false},
 
 handle_info(#'basic.ack'{delivery_tag = Seq, multiple = Multiple},
             State = #state{config = #shovel{ack_mode = on_confirm}}) ->
-    {noreply, confirm_to_inbound(#'basic.ack'{delivery_tag = _, multiple = _},
-                                 Seq, Multiple, State)};
+    {noreply, confirm_to_inbound(
+                fun (DTag, Multiple) ->
+                        #'basic.ack'{delivery_tag = DTag, multiple = Multiple}
+                end, Seq, Multiple, State)};
 
 handle_info(#'basic.nack'{delivery_tag = Seq, multiple = Multiple},
             State = #state{config = #shovel{ack_mode = on_confirm}}) ->
-    {noreply, confirm_to_inbound(#'basic.nack'{delivery_tag = _, multiple = _},
-                                 Seq, Multiple, State)};
+    {noreply, confirm_to_inbound(
+                fun (DTag, Multiple) ->
+                        #'basic.nack'{delivery_tag = DTag, multiple = Multiple}
+                end, Seq, Multiple, State)};
+
+handle_info(#'basic.cancel'{}, State = #state{name = Name}) ->
+    rabbit_log:warning("Shovel ~p received 'basic.cancel' from the broker~n",
+                       [Name]),
+    {stop, {shutdown, restart}, State};
 
 handle_info({'EXIT', InboundConn, Reason},
             State = #state{inbound_conn = InboundConn}) ->
@@ -148,10 +157,10 @@ terminate(Reason, #state{inbound_conn = undefined, inbound_ch = undefined,
     rabbit_shovel_status:report(Name, {terminated, Reason}),
     ok;
 terminate(Reason, State) ->
-    catch amqp_channel:close(State#state.inbound_ch),
-    catch amqp_connection:close(State#state.inbound_conn),
-    catch amqp_channel:close(State#state.outbound_ch),
-    catch amqp_connection:close(State#state.outbound_conn),
+    catch amqp_connection:close(State#state.inbound_conn,
+                                ?MAX_CONNECTION_CLOSE_TIMEOUT),
+    catch amqp_connection:close(State#state.outbound_conn,
+                                ?MAX_CONNECTION_CLOSE_TIMEOUT),
     rabbit_shovel_status:report(State#state.name, {terminated, Reason}),
     ok.
 
