@@ -11,7 +11,7 @@
 %% The Original Code is RabbitMQ.
 %%
 %% The Initial Developer of the Original Code is VMware, Inc.
-%% Copyright (c) 2007-2012 VMware, Inc.  All rights reserved.
+%% Copyright (c) 2007-2013 VMware, Inc.  All rights reserved.
 %%
 
 -module(rabbit_stomp_processor).
@@ -32,9 +32,8 @@
                 config, dest_queues, reply_queues, frame_transformer,
                 adapter_info, send_fun, ssl_login_name}).
 
--record(subscription, {dest_hdr, channel, multi_ack, description}).
+-record(subscription, {dest_hdr, channel, ack_mode, multi_ack, description}).
 
--define(SUPPORTED_VERSIONS, ["1.0", "1.1"]).
 -define(FLUSH_TIMEOUT, 60000).
 
 %%----------------------------------------------------------------------------
@@ -105,7 +104,7 @@ handle_cast(_Request, State = #state{channel = none,
                                       implicit_connect = false}}) ->
     {noreply,
      send_error("Illegal command",
-                "You must log in using CONNECT first\n",
+                "You must log in using CONNECT first",
                 State),
      hibernate};
 
@@ -138,6 +137,8 @@ handle_info(#'basic.ack'{delivery_tag = Tag, multiple = IsMulti}, State) ->
 handle_info({Delivery = #'basic.deliver'{},
              #amqp_msg{props = Props, payload = Payload}}, State) ->
     {noreply, send_delivery(Delivery, Props, Payload, State), hibernate};
+handle_info(#'basic.cancel'{consumer_tag = Ctag}, State) ->
+    {noreply, server_cancel_consumer(Ctag, State), hibernate};
 handle_info({'EXIT', Conn,
              {shutdown, {server_initiated_close, Code, Explanation}}},
             State = #state{connection = Conn}) ->
@@ -150,7 +151,6 @@ handle_info({inet_reply, _, ok}, State) ->
 handle_info({bump_credit, Msg}, State) ->
     credit_flow:handle_bump_msg(Msg),
     {noreply, State, hibernate};
-
 handle_info({inet_reply, _, Status}, State) ->
     {stop, Status, State}.
 
@@ -161,7 +161,7 @@ process_request(ProcessFun, SuccessFun, State) ->
                  {server_initiated_close, ReplyCode, Explanation}}, _}} ->
                   amqp_death(ReplyCode, Explanation, State);
               {'EXIT', Reason} ->
-                  priv_error("Processing error", "Processing error\n",
+                  priv_error("Processing error", "Processing error",
                               Reason, State);
               Result ->
                   Result
@@ -192,16 +192,16 @@ process_connect(Implicit, Frame,
                   {ok, Version} ->
                       FT = frame_transformer(Version),
                       Frame1 = FT(Frame),
-                      {Username, Creds} = creds(Frame1, SSLLoginName, Config),
+                      {Username, Passwd} = creds(Frame1, SSLLoginName, Config),
                       {ok, DefaultVHost} =
                           application:get_env(rabbit, default_vhost),
-                      {ProtoName, _} = AdapterInfo#adapter_info.protocol,
+                      {ProtoName, _} = AdapterInfo#amqp_adapter_info.protocol,
                       Res = do_login(
-                              Username, Creds,
+                              Username, Passwd,
                               login_header(Frame1, ?HEADER_HOST, DefaultVHost),
                               login_header(Frame1, ?HEADER_HEART_BEAT, "0,0"),
-                                AdapterInfo#adapter_info{
-                                  protocol = {ProtoName, Version}}, Version,
+                              AdapterInfo#amqp_adapter_info{
+                                protocol = {ProtoName, Version}}, Version,
                               StateN#state{frame_transformer = FT}),
                       case {Res, Implicit} of
                           {{ok, _, StateN1}, implicit} -> ok(StateN1);
@@ -209,7 +209,7 @@ process_connect(Implicit, Frame,
                       end;
                   {error, no_common_version} ->
                       error("Version mismatch",
-                            "Supported versions are ~s\n",
+                            "Supported versions are ~s~n",
                             [string:join(?SUPPORTED_VERSIONS, ",")],
                             StateN)
               end
@@ -220,12 +220,11 @@ process_connect(Implicit, Frame,
 creds(Frame, SSLLoginName,
       #stomp_configuration{default_login    = DefLogin,
                            default_passcode = DefPasscode}) ->
-    PasswordCreds = {login_header(Frame, ?HEADER_LOGIN, DefLogin),
-                     [{password, login_header(
-                                   Frame, ?HEADER_PASSCODE, DefPasscode)}]},
+    PasswordCreds = {login_header(Frame, ?HEADER_LOGIN,    DefLogin),
+                     login_header(Frame, ?HEADER_PASSCODE, DefPasscode)},
     case {rabbit_stomp_frame:header(Frame, ?HEADER_LOGIN), SSLLoginName} of
         {not_found, none}    -> PasswordCreds;
-        {not_found, SSLName} -> {SSLName, []};
+        {not_found, SSLName} -> {SSLName, none};
         _                    -> PasswordCreds
     end.
 
@@ -297,7 +296,7 @@ handle_frame("ABORT", Frame, State) ->
 
 handle_frame(Command, _Frame, State) ->
     error("Bad command",
-          "Could not interpret command ~p\n",
+          "Could not interpret command ~p~n",
           [Command],
           State).
 
@@ -306,39 +305,68 @@ handle_frame(Command, _Frame, State) ->
 %%----------------------------------------------------------------------------
 
 ack_action(Command, Frame,
-           State = #state{subscriptions = Subs}, MethodFun) ->
-    case rabbit_stomp_frame:header(Frame, ?HEADER_MESSAGE_ID) of
-        {ok, IdStr} ->
-            case rabbit_stomp_util:parse_message_id(IdStr) of
+           State = #state{subscriptions = Subs,
+                          version       = Version}, MethodFun) ->
+    AckHeader = rabbit_stomp_util:ack_header_name(Version),
+    case rabbit_stomp_frame:header(Frame, AckHeader) of
+        {ok, AckValue} ->
+            case rabbit_stomp_util:parse_message_id(AckValue) of
                 {ok, {ConsumerTag, _SessionId, DeliveryTag}} ->
-                    Subscription = #subscription{channel = SubChannel}
-                        = dict:fetch(ConsumerTag, Subs),
-                    Method = MethodFun(DeliveryTag, Subscription),
-                    case transactional(Frame) of
-                        {yes, Transaction} ->
-                            extend_transaction(Transaction,
-                                               {SubChannel, Method},
-                                               State);
-                        no ->
-                            amqp_channel:call(SubChannel, Method),
-                            ok(State)
+                    case dict:find(ConsumerTag, Subs) of
+                        {ok, Sub = #subscription{channel = SubChannel}} ->
+                            Method = MethodFun(DeliveryTag, Sub),
+                            case transactional(Frame) of
+                                {yes, Transaction} ->
+                                    extend_transaction(Transaction,
+                                                       {SubChannel, Method},
+                                                       State);
+                                no ->
+                                    amqp_channel:call(SubChannel, Method),
+                                    ok(State)
+                            end;
+                        error ->
+                            error("Subscription not found",
+                                  "Message with id ~p has no subscription",
+                                  [AckValue],
+                                  State)
                     end;
                 _ ->
-                   error("Invalid message-id",
-                         "~p must include a valid 'message-id' header\n",
-                         [Command],
+                   error("Invalid header",
+                         "~p must include a valid ~p header~n",
+                         [Command, AckHeader],
                          State)
             end;
         not_found ->
-            error("Missing message-id",
-                  "~p must include a 'message-id' header\n",
-                  [Command],
+            error("Missing header",
+                  "~p must include the ~p header~n",
+                  [Command, AckHeader],
                   State)
     end.
 
 %%----------------------------------------------------------------------------
 %% Internal helpers for processing frames callbacks
 %%----------------------------------------------------------------------------
+server_cancel_consumer(ConsumerTag, State = #state{subscriptions = Subs}) ->
+    case dict:find(ConsumerTag, Subs) of
+        error ->
+            error("Server cancelled unknown subscription",
+                  "Consumer tag ~p is not associated with a subscription.~n",
+                  [ConsumerTag],
+                  State);
+        {ok, Subscription = #subscription{description = Description}} ->
+            Id = case rabbit_stomp_util:tag_to_id(ConsumerTag) of
+                     {ok,    {_, Id1}} -> Id1;
+                     {error, {_, Id1}} -> "Unknown[" ++ Id1 ++ "]"
+                 end,
+            send_error_frame("Server cancelled subscription",
+                             [{?HEADER_SUBSCRIPTION, Id}],
+                             "The server has canceled a subscription.~n"
+                             "No more messages will be delivered for ~p.~n",
+                             [Description],
+                             State),
+            tidy_canceled_subscription(ConsumerTag, Subscription,
+                                       #stomp_frame{}, State)
+    end.
 
 cancel_subscription({error, invalid_prefix}, _Frame, State) ->
     error("Invalid id",
@@ -348,55 +376,58 @@ cancel_subscription({error, invalid_prefix}, _Frame, State) ->
 
 cancel_subscription({error, _}, _Frame, State) ->
     error("Missing destination or id",
-          "UNSUBSCRIBE must include a 'destination' or 'id' header\n",
+          "UNSUBSCRIBE must include a 'destination' or 'id' header",
           State);
 
 cancel_subscription({ok, ConsumerTag, Description}, Frame,
-                    State = #state{channel       = MainChannel,
-                                   subscriptions = Subs}) ->
+                    State = #state{subscriptions = Subs}) ->
     case dict:find(ConsumerTag, Subs) of
         error ->
             error("No subscription found",
-                  "UNSUBSCRIBE must refer to an existing subscription.\n"
-                  "Subscription to ~p not found.\n",
+                  "UNSUBSCRIBE must refer to an existing subscription.~n"
+                  "Subscription to ~p not found.~n",
                   [Description],
                   State);
-        {ok, #subscription{dest_hdr = DestHdr, channel = SubChannel}} ->
+        {ok, Subscription = #subscription{channel = SubChannel,
+                                          description = Descr}} ->
             case amqp_channel:call(SubChannel,
                                    #'basic.cancel'{
                                      consumer_tag = ConsumerTag}) of
                 #'basic.cancel_ok'{consumer_tag = ConsumerTag} ->
-                    ok = ensure_subchannel_closed(SubChannel, MainChannel),
-                    NewSubs = dict:erase(ConsumerTag, Subs),
-                    maybe_delete_durable_sub(DestHdr, Frame,
-                                             State#state{
-                                               subscriptions = NewSubs});
+                    tidy_canceled_subscription(ConsumerTag, Subscription,
+                                               Frame, State);
                 _ ->
                     error("Failed to cancel subscription",
-                          "UNSUBSCRIBE to ~p failed.\n",
-                          [Description],
+                          "UNSUBSCRIBE to ~p failed.~n",
+                          [Descr],
                           State)
             end
     end.
 
-maybe_delete_durable_sub(DestHdr, Frame, State = #state{channel = Channel}) ->
-    case rabbit_stomp_util:parse_destination(DestHdr) of
-        {ok, {topic, Name}} ->
-            case rabbit_stomp_frame:boolean_header(Frame,
-                                                   ?HEADER_PERSISTENT, false) of
-                true ->
-                    {ok, Id} = rabbit_stomp_frame:header(Frame, ?HEADER_ID),
-                    QName =
-                        rabbit_stomp_util:durable_subscription_queue(Name, Id),
-                    amqp_channel:call(Channel, #'queue.delete'{queue  = QName,
-                                                               nowait = false}),
-                    ok(State);
-                false ->
-                    ok(State)
-            end;
-        _ ->
+tidy_canceled_subscription(ConsumerTag, #subscription{dest_hdr = DestHdr,
+                                                      channel = SubChannel},
+                           Frame, State = #state{channel = MainChannel,
+                                                 subscriptions = Subs}) ->
+    ok = ensure_subchannel_closed(SubChannel, MainChannel),
+    Subs1 = dict:erase(ConsumerTag, Subs),
+    {ok, Dest} = rabbit_stomp_util:parse_destination(DestHdr),
+    maybe_delete_durable_sub(Dest, Frame, State#state{subscriptions = Subs1}).
+
+maybe_delete_durable_sub({topic, Name}, Frame,
+                         State = #state{channel = Channel}) ->
+    case rabbit_stomp_frame:boolean_header(Frame,
+                                           ?HEADER_PERSISTENT, false) of
+        true ->
+            {ok, Id} = rabbit_stomp_frame:header(Frame, ?HEADER_ID),
+            QName = rabbit_stomp_util:durable_subscription_queue(Name, Id),
+            amqp_channel:call(Channel, #'queue.delete'{queue  = QName,
+                                                       nowait = false}),
+            ok(State);
+        false ->
             ok(State)
-    end.
+    end;
+maybe_delete_durable_sub(_Destination, _Frame, State) ->
+    ok(State).
 
 ensure_subchannel_closed(SubChannel, MainChannel)
   when SubChannel == MainChannel ->
@@ -414,21 +445,20 @@ with_destination(Command, Frame, State, Fun) ->
                     Fun(Destination, DestHdr, Frame, State);
                 {error, {invalid_destination, Type, Content}} ->
                     error("Invalid destination",
-                          "'~s' is not a valid ~p destination\n",
+                          "'~s' is not a valid ~p destination~n",
                           [Content, Type],
                           State);
                 {error, {unknown_destination, Content}} ->
                     error("Unknown destination",
-                          "'~s' is not a valid destination.\n" ++
-                              "Valid destination types are: " ++
-                   string:join(rabbit_stomp_util:valid_dest_prefixes(),", ") ++
-                              ".\n",
-                          [Content],
-                          State)
+                          "'~s' is not a valid destination.~n"
+                          "Valid destination types are: ~s.~n",
+                          [Content,
+                           string:join(rabbit_stomp_util:valid_dest_prefixes(),
+                                       ", ")], State)
             end;
         not_found ->
             error("Missing destination",
-                  "~p must include a 'destination' header\n",
+                  "~p must include a 'destination' header~n",
                   [Command],
                   State)
     end.
@@ -437,7 +467,7 @@ without_headers([Hdr | Hdrs], Command, Frame, State, Fun) ->
     case rabbit_stomp_frame:header(Frame, Hdr) of
         {ok, _} ->
             error("Invalid header",
-                  "'~s' is not allowed on '~s'.\n",
+                  "'~s' is not allowed on '~s'.~n",
                   [Hdr, Command],
                   State);
         not_found ->
@@ -447,50 +477,47 @@ without_headers([], Command, Frame, State, Fun) ->
     Fun(Command, Frame, State).
 
 do_login(undefined, _, _, _, _, _, State) ->
-    error("Bad CONNECT", "Missing login or passcode header(s)\n", State);
-do_login(Username, Creds, VirtualHost, Heartbeat, AdapterInfo, Version,
+    error("Bad CONNECT", "Missing login or passcode header(s)", State);
+do_login(Username, Passwd, VirtualHost, Heartbeat, AdapterInfo, Version,
          State) ->
-    case rabbit_access_control:check_user_login(Username, Creds) of
-        {ok, _User} ->
-            case amqp_connection:start(
-                   #amqp_params_direct{username     = Username,
-                                       virtual_host = VirtualHost,
-                                       adapter_info = AdapterInfo}) of
-                {ok, Connection} ->
-                    link(Connection),
-                    {ok, Channel} = amqp_connection:open_channel(Connection),
-                    SessionId =
-                        rabbit_guid:string(rabbit_guid:gen_secure(), "session"),
-                    {{SendTimeout, ReceiveTimeout}, State1} =
-                        ensure_heartbeats(Heartbeat, State),
-                    ok("CONNECTED",
-                       [{?HEADER_SESSION, SessionId},
-                        {?HEADER_HEART_BEAT,
-                         io_lib:format("~B,~B", [SendTimeout, ReceiveTimeout])},
-                        {?HEADER_SERVER, server_header()},
-                        {?HEADER_VERSION, Version}],
-                       "",
-                       State1#state{session_id = SessionId,
-                                    channel    = Channel,
-                                    connection = Connection});
-                {error, auth_failure} ->
-                    rabbit_log:error("STOMP login failed - auth_failure "
-                                     "(user vanished)~n"),
-                    error("Bad CONNECT", "Authentication failure\n", State);
-                {error, access_refused} ->
-                    rabbit_log:warning("STOMP login failed - access_refused "
-                                       "(vhost access not allowed)~n"),
-                    error("Bad CONNECT", "Authentication failure\n", State)
-            end;
-        {refused, Msg, Args} ->
-            rabbit_log:warning("STOMP login failed: " ++ Msg ++ "\n", Args),
-            error("Bad CONNECT", "Authentication failure\n", State)
+    case amqp_connection:start(
+           #amqp_params_direct{username     = Username,
+                               password     = Passwd,
+                               virtual_host = VirtualHost,
+                               adapter_info = AdapterInfo}) of
+        {ok, Connection} ->
+            link(Connection),
+            {ok, Channel} = amqp_connection:open_channel(Connection),
+            SessionId = rabbit_guid:string(rabbit_guid:gen_secure(), "session"),
+            {{SendTimeout, ReceiveTimeout}, State1} =
+                ensure_heartbeats(Heartbeat, State),
+            ok("CONNECTED",
+               [{?HEADER_SESSION, SessionId},
+                {?HEADER_HEART_BEAT,
+                 io_lib:format("~B,~B", [SendTimeout, ReceiveTimeout])},
+                {?HEADER_SERVER, server_header()},
+                {?HEADER_VERSION, Version}],
+               "",
+               State1#state{session_id = SessionId,
+                            channel    = Channel,
+                            connection = Connection,
+                            version    = Version});
+        {error, auth_failure} ->
+            rabbit_log:warning("STOMP login failed for user ~p~n",
+                               [binary_to_list(Username)]),
+            error("Bad CONNECT", "Access refused for user '" ++
+                  binary_to_list(Username) ++ "'~n", [], State);
+        {error, access_refused} ->
+            rabbit_log:warning("STOMP login failed - access_refused "
+                               "(vhost access not allowed)~n"),
+            error("Bad CONNECT", "Virtual host '" ++
+                                 binary_to_list(VirtualHost) ++
+                                 "' access denied", State)
     end.
 
 server_header() ->
-    Props = rabbit_reader:server_properties(?PROTOCOL),
-    {_, Product} = rabbit_misc:table_lookup(Props, <<"product">>),
-    {_, Version} = rabbit_misc:table_lookup(Props, <<"version">>),
+    {ok, Product} = application:get_key(rabbit, id),
+    {ok, Version} = application:get_key(rabbit, vsn),
     rabbit_misc:format("~s/~s", [Product, Version]).
 
 do_subscribe(Destination, DestHdr, Frame,
@@ -499,8 +526,8 @@ do_subscribe(Destination, DestHdr, Frame,
                             connection    = Connection,
                             channel       = MainChannel}) ->
     Prefetch =
-        rabbit_stomp_frame:integer_header(Frame, "prefetch-count", undefined),
-
+        rabbit_stomp_frame:integer_header(Frame, ?HEADER_PREFETCH_COUNT,
+                                          undefined),
     Channel = case Prefetch of
                   undefined ->
                       MainChannel;
@@ -535,6 +562,7 @@ do_subscribe(Destination, DestHdr, Frame,
                        dict:store(ConsumerTag,
                                   #subscription{dest_hdr    = DestHdr,
                                                 channel     = Channel,
+                                                ack_mode    = AckMode,
                                                 multi_ack   = IsMulti,
                                                 description = Description},
                                   Subs),
@@ -590,13 +618,14 @@ negotiate_version(Frame) ->
 send_delivery(Delivery = #'basic.deliver'{consumer_tag = ConsumerTag},
               Properties, Body,
               State = #state{session_id    = SessionId,
-                             subscriptions = Subs}) ->
+                             subscriptions = Subs,
+                             version       = Version}) ->
     case dict:find(ConsumerTag, Subs) of
-        {ok, #subscription{}} ->
+        {ok, #subscription{ack_mode = AckMode}} ->
             send_frame(
               "MESSAGE",
-              rabbit_stomp_util:message_headers(SessionId, Delivery,
-                                                Properties),
+              rabbit_stomp_util:headers(SessionId, Delivery, Properties,
+                                        AckMode, Version),
               Body,
               State);
         error ->
@@ -659,7 +688,7 @@ ensure_reply_queue(TempQueueId, State = #state{channel       = Channel,
                                                subscriptions = Subs}) ->
     case dict:find(TempQueueId, RQS) of
         {ok, RQ} ->
-            {reply_to_destination(RQ), State};
+            {binary_to_list(RQ), State};
         error ->
             #'queue.declare_ok'{queue = Queue} =
                 amqp_channel:call(Channel,
@@ -676,7 +705,7 @@ ensure_reply_queue(TempQueueId, State = #state{channel       = Channel,
                                          nowait       = false},
                                        self()),
 
-            Destination = reply_to_destination(Queue),
+            Destination = binary_to_list(Queue),
 
             %% synthesise a subscription to the reply queue destination
             Subs1 = dict:store(ConsumerTag,
@@ -690,9 +719,6 @@ ensure_reply_queue(TempQueueId, State = #state{channel       = Channel,
                             subscriptions = Subs1}}
     end.
 
-reply_to_destination(Queue) ->
-    ?REPLY_QUEUE_PREFIX ++ binary_to_list(Queue).
-
 %%----------------------------------------------------------------------------
 %% Receipt Handling
 %%----------------------------------------------------------------------------
@@ -767,7 +793,7 @@ accumulate_receipts1(DeliveryTag, {_Key, Value, PR}, Acc) ->
 %%----------------------------------------------------------------------------
 
 transactional(Frame) ->
-    case rabbit_stomp_frame:header(Frame, "transaction") of
+    case rabbit_stomp_frame:header(Frame, ?HEADER_TRANSACTION) of
         {ok, Transaction} -> {yes, Transaction};
         not_found         -> no
     end.
@@ -778,7 +804,7 @@ transactional_action(Frame, Name, Fun, State) ->
             Fun(Transaction, State);
         no ->
             error("Missing transaction",
-                  "~p must include a 'transaction' header\n",
+                  "~p must include a 'transaction' header~n",
                   [Name],
                   State)
     end.
@@ -787,7 +813,7 @@ with_transaction(Transaction, State, Fun) ->
     case get({transaction, Transaction}) of
         undefined ->
             error("Bad transaction",
-                  "Invalid transaction identifier: ~p\n",
+                  "Invalid transaction identifier: ~p~n",
                   [Transaction],
                   State);
         Actions ->
@@ -889,7 +915,7 @@ ensure_queue(_, {queue, Name}, _Frame, Channel, DestQs) ->
               end,
     {ok, Queue, DestQs1};
 ensure_queue(subscribe, {topic, Name}, Frame, Channel, DestQs) ->
-    %% Create queue for SUBSCRIBE on /topic destinations Queues are
+    %% Create queue for SUBSCRIBE on /topic destinations. Queues are
     %% anonymous, auto_delete and exclusive for transient
     %% subscriptions. Durable subscriptions get shared, named, durable
     %% queues.
@@ -903,9 +929,7 @@ ensure_queue(subscribe, {topic, Name}, Frame, Channel, DestQs) ->
             false ->
                 #'queue.declare'{auto_delete = true, exclusive = true}
         end,
-
-    #'queue.declare_ok'{queue = Queue} =
-        amqp_channel:call(Channel, Method),
+    #'queue.declare_ok'{queue = Queue} = amqp_channel:call(Channel, Method),
     {ok, Queue, DestQs};
 ensure_queue(send, {topic, _}, _Frame, _Channel, DestQs) ->
     %% Don't create queues on SEND for /topic destinations
@@ -940,7 +964,7 @@ ok(Command, Headers, BodyFragments, State) ->
                       body_iolist = BodyFragments}, State}.
 
 amqp_death(ReplyCode, Explanation, State) ->
-    ErrorName = ?PROTOCOL:amqp_exception(ReplyCode),
+    ErrorName = amqp_connection:error_atom(ReplyCode),
     ErrorDesc = rabbit_misc:format("~s~n", [Explanation]),
     log_error(ErrorName, ErrorDesc, none),
     {stop, normal, send_error(atom_to_list(ErrorName), ErrorDesc, State)}.
@@ -979,11 +1003,19 @@ send_frame(Frame, State = #state{send_fun = SendFun}) ->
     SendFun(async, rabbit_stomp_frame:serialize(Frame)),
     State.
 
-send_error(Message, Detail, State) ->
+send_error_frame(Message, ExtraHeaders, Format, Args, State) ->
+    send_error_frame(Message, ExtraHeaders, rabbit_misc:format(Format, Args),
+                     State).
+
+send_error_frame(Message, ExtraHeaders, Detail, State) ->
     send_frame("ERROR", [{"message", Message},
                          {"content-type", "text/plain"},
-                         {"version", string:join(?SUPPORTED_VERSIONS, ",")}],
-                         Detail, State).
+                         {"version", string:join(?SUPPORTED_VERSIONS, ",")}] ++
+                        ExtraHeaders,
+                        Detail, State).
+
+send_error(Message, Detail, State) ->
+    send_error_frame(Message, [], Detail, State).
 
 send_error(Message, Format, Args, State) ->
     send_error(Message, rabbit_misc:format(Format, Args), State).
