@@ -11,7 +11,7 @@
 %% The Original Code is RabbitMQ.
 %%
 %% The Initial Developer of the Original Code is VMware, Inc.
-%% Copyright (c) 2007-2012 VMware, Inc.  All rights reserved.
+%% Copyright (c) 2007-2013 VMware, Inc.  All rights reserved.
 %%
 
 -module(rabbit_stomp_test_util).
@@ -19,6 +19,7 @@
 -include_lib("eunit/include/eunit.hrl").
 -include_lib("amqp_client/include/amqp_client.hrl").
 -include("rabbit_stomp_frame.hrl").
+-include("rabbit_stomp_prefixes.hrl").
 
 %%--------------------------------------------------------------------
 %% Header Processing Tests
@@ -36,35 +37,35 @@ message_properties_test() ->
                 {"priority", "1"},
                 {"correlation-id", "123"},
                 {"reply-to", "something"},
+                {"expiration", "my-expiration"},
                 {"amqp-message-id", "M123"},
+                {"timestamp", "123456"},
+                {"type", "freshly-squeezed"},
+                {"user-id", "joe"},
+                {"app-id", "joe's app"},
                 {"str", "foo"},
                 {"int", "123"}
               ],
 
     #'P_basic'{
-                content_type = <<"text/plain">>,
+                content_type     = <<"text/plain">>,
                 content_encoding = <<"UTF-8">>,
-                delivery_mode = 2,
-                priority = 1,
-                correlation_id = <<"123">>,
-                reply_to = <<"something">>,
-                message_id = <<"M123">>,
-                headers = [
-                           {<<"str">>, longstr, <<"foo">>},
-                           {<<"int">>, longstr, <<"123">>}]
-                } =
+                delivery_mode    = 2,
+                priority         = 1,
+                correlation_id   = <<"123">>,
+                reply_to         = <<"something">>,
+                expiration       = <<"my-expiration">>,
+                message_id       = <<"M123">>,
+                timestamp        = 123456,
+                type             = <<"freshly-squeezed">>,
+                user_id          = <<"joe">>,
+                app_id           = <<"joe's app">>,
+                headers          = [{<<"str">>, longstr, <<"foo">>},
+                                    {<<"int">>, longstr, <<"123">>}]
+              } =
         rabbit_stomp_util:message_properties(#stomp_frame{headers = Headers}).
 
 message_headers_test() ->
-    Destination = "/queue/foo",
-    SessionId = "1234567",
-
-    Delivery = #'basic.deliver'{
-      consumer_tag = <<"Q_123">>,
-      delivery_tag = 123,
-      exchange = <<"">>,
-      routing_key = <<"foo">>},
-
     Properties = #'P_basic'{
       headers          = [{<<"str">>, longstr, <<"foo">>},
                           {<<"int">>, signedint, 123}],
@@ -74,21 +75,27 @@ message_headers_test() ->
       priority         = 1,
       correlation_id   = 123,
       reply_to         = <<"something">>,
-      message_id       = <<"M123">>},
+      message_id       = <<"M123">>,
+      timestamp        = 123456,
+      type             = <<"freshly-squeezed">>,
+      user_id          = <<"joe">>,
+      app_id           = <<"joe's app">>},
 
-    Headers = rabbit_stomp_util:message_headers(SessionId, Delivery,
-                                                Properties),
+    Headers = rabbit_stomp_util:message_headers(Properties),
 
     Expected = [
-                {"destination", Destination},
-                {"message-id", [<<"Q_123">>, "@@", SessionId, "@@", "123"]},
                 {"content-type", "text/plain"},
                 {"content-encoding", "UTF-8"},
                 {"persistent", "true"},
                 {"priority", "1"},
                 {"correlation-id", "123"},
                 {"reply-to", "something"},
+                {"expiration", "my-expiration"},
                 {"amqp-message-id", "M123"},
+                {"timestamp", "123456"},
+                {"type", "freshly-squeezed"},
+                {"user-id", "joe"},
+                {"app-id", "joe's app"},
                 {"str", "foo"},
                 {"int", "123"}
                ],
@@ -96,9 +103,6 @@ message_headers_test() ->
     [] = lists:subtract(Headers, Expected).
 
 minimal_message_headers_with_no_custom_test() ->
-    Destination = "/queue/foo",
-    SessionId = "1234567",
-
     Delivery = #'basic.deliver'{
       consumer_tag = <<"Q_123">>,
       delivery_tag = 123,
@@ -107,11 +111,8 @@ minimal_message_headers_with_no_custom_test() ->
 
     Properties = #'P_basic'{},
 
-    Headers = rabbit_stomp_util:message_headers(SessionId, Delivery,
-                                                Properties),
+    Headers = rabbit_stomp_util:message_headers(Properties),
     Expected = [
-                {"destination", Destination},
-                {"message-id", [<<"Q_123">>, "@@", SessionId, "@@", "123"]},
                 {"content-type", "text/plain"},
                 {"content-encoding", "UTF-8"},
                 {"amqp-message-id", "M123"}
@@ -119,6 +120,30 @@ minimal_message_headers_with_no_custom_test() ->
 
     [] = lists:subtract(Headers, Expected).
 
+headers_post_process_test() ->
+    Headers  = [{"header1", "1"},
+                {"header2", "12"},
+                {"reply-to", "something"}],
+    Expected = [{"header1", "1"},
+                {"header2", "12"},
+                {"reply-to", "/reply-queue/something"}],
+    [] = lists:subtract(
+           rabbit_stomp_util:headers_post_process(Headers), Expected).
+
+headers_post_process_noop_replyto_test() ->
+    [begin
+         Headers = [{"reply-to", Prefix ++ "/something"}],
+         Headers = rabbit_stomp_util:headers_post_process(Headers)
+     end || Prefix <- ?VALID_DEST_PREFIXES, Prefix /= ?TEMP_QUEUE_PREFIX].
+
+headers_post_process_noop2_test() ->
+    Headers  = [{"header1", "1"},
+                {"header2", "12"}],
+    Expected = [{"header1", "1"},
+                {"header2", "12"}],
+    [] = lists:subtract(
+           rabbit_stomp_util:headers_post_process(Headers), Expected).
+
 negotiate_version_both_empty_test() ->
     {error, no_common_version} = rabbit_stomp_util:negotiate_version([],[]).
 
